. ${PSScriptRoot}\def_copies.ps1

# Define variables
${ACTIVE_MQ_DEFAULT} = "C:\netiq\idm\apps\activemq"
${JAVA_DEFAULT} = "C:\netiq\idm\apps\jre"
${POSTGRESQL_ADMIN_DEFAULT} = "postgres"
${POSTGRESQL_PATH_DEFAULT} = "C:\netiq\idm\apps\postgres"
${POSTGRESQL_PORT_DEFAULT} = "5432"
${POSTGRESQL_SERVER_DEFAULT} = "localhost"
${TOMCAT_DEFAULT} = "C:\netiq\idm\apps\tomcat"

${DIR_COMPRESSED} = "${PSScriptRoot}\compressed"

${FILE_ACTIVE_MQ} = "${DIR_COMPRESSED}\apache-activemq-${ACTIVE_MQ_VERSION_TARGET}-bin.zip"
${FILE_JAVA} = "${DIR_COMPRESSED}\zulu${JAVA_DOWNLOAD_CODE}-win_x64.zip"
${FILE_POSTGRESQL} = "${DIR_COMPRESSED}\postgresql-${POSTGRESQL_VERSION_TARGET}-windows-x64.exe"
${FILE_TOMCAT} = "${DIR_COMPRESSED}\apache-tomcat-${TOMCAT_VERSION_TARGET}-windows-x64.zip"

${global:ALREADY_OBTAINED_JAVA_LOC} = ${FALSE}

${TIMESTAMP} = (Get-Date -Format "yyyyMMdd-HHmmss")

${COMPARE_CODE_EQ} = 0
${COMPARE_CODE_LT} = 1
${COMPARE_CODE_GT} = 2

${global:SUMMARY_REGISTRATION}=@()
${SUMMARY_CODE_COMPLETED} = 0
${SUMMARY_CODE_SKIPPED} = 1
${SUMMARY_CODE_MISSED} = 2
${SUMMARY_CODE_FAILED} = 3

Add-Type -AssemblyName System.IO.Compression.FileSystem

# Define functions
Function Get-PathValidity ([string]${base}, [string[]]${array}) {
  ${valid} = ${TRUE}
  ForEach (${path} in ${array}) {
    If (-not (Test-Path ${base}\${path})) {
      Write-Information "    Expected to find '${base}\${path}'" -InformationAction Continue
      ${valid}=${FALSE}
    }
  }
  return ${valid}
}

Function Get-ValidJavaPath () {
  If (-not ${global:ALREADY_OBTAINED_JAVA_LOC}) {
    ${global:ALREADY_OBTAINED_JAVA_LOC} = ${TRUE}
    ${input_invalid} = ${TRUE}
    while (${input_invalid}) {
      ${java_loc} = Read-Host "  Where is your Java (JRE) located? [${java_loc_final}]"
      If ("${java_loc}" -ne "") { ${java_loc_final} = ${java_loc} }
      ${result} = (Get-PathValidity ${java_loc_final} ${JAVA_VALIDITY})
      If ($result -eq ${TRUE}) {
        ${input_invalid} = ${FALSE}
      }
    }
  }
}

Function Parse-Version ([string]${version}) {
  return (${version} -replace '\.', ' ' -replace '-', ' ' -replace '_', ' ').split(" ")
}

Function Compare-Versions ([string]${A}, [string]${B}) {
  ${version_A} = Parse-Version ${A}
  ${version_B} = Parse-Version ${B}

  ${max} = ${version_B}.count
  For ((${i} = 0); ${i} -lt ${max}; ${i}++) {
    try {
      ${actual} = [int]::Parse(${version_A}[${i}])
      ${expected} = [int]::Parse(${version_B}[${i}])
    } catch [System.Management.Automation.MethodInvocationException] {
      # Compare as strings in case of an error
      ${actual} = ${version_A}[${i}]
      ${expected} = ${version_B}[${i}]
    }
    If ( ${actual} -lt ${expected} ) {
      return ${COMPARE_CODE_LT}
    } ElseIf ( ${actual} -gt ${expected} ) {
      return ${COMPARE_CODE_GT}
    }
  }
  return ${COMPARE_CODE_EQ}
}

Function Compare-VersionsWithResult ([string]${name}, [string]${actual}, [string]${expected}) {
  ${comparison} = Compare-Versions ${actual} ${expected}
  If ( ${comparison} -eq ${COMPARE_CODE_LT} ) {
    Write-Host "  Expected ${name} version ${expected}, but instead found the older ${actual}"
  } ElseIf ( ${comparison} -eq ${COMPARE_CODE_GT} ) {
    Write-Host "  Your ${name} version (${actual}) surpasses the target (${expected})`n    We cannot guarantee your version will work, but it might."
  } Else { # If ( ${comparison} -eq ${COMPARE_CODE_EQ} )
    Write-Host "  Your ${name} version (${expected}) appears to be valid."
  }
}

Function Wait-StopProcesses([string]${component}, [string]${search}) {
  ${private:process_all} = (Get-WmiObject -ClassName Win32_Process -Filter "CommandLine LIKE '%${search}%'")
  While (${process_all} -ne ${null}) {
    ${private:do_please} = Read-Host "    It appears that ${component} is still running.`n      May we forcefully stop it [y/N]?"
    If ("${do_please}" -eq "y" -Or "${do_please}" -eq "Y") {
      ForEach (${process} in ${process_all}) {
        Write-Host "    Stopping `"$(${process}.Name)`":"
        ${null} = ${process}.Terminate()
        Start-Sleep 3
      }
    } Else {
      Read-Host "    Go ahead and stop your ${component} instances.`n      When the ${component} instances have completely stopped, press ENTER."
    }
    ${private:process_all} = (Get-WmiObject -ClassName Win32_Process -Filter "CommandLine LIKE '%${search}%'")
  }
  Write-Host "    It appears no instances of ${component} are running. Proceeding ..."
}

Function Wait-StopServices([string]${component}, [string]${search}) {
  ${private:service_all} = (Get-Service -DisplayName "*${search}*" | Where-Object {$_.Status -eq "Running"})
  While (${service_all} -ne ${null}) {
    ${private:do_please} = Read-Host "    It appears that ${component} is still running.`n      May we forcefully stop it [y/N]?"
    If ("${do_please}" -eq "y" -Or "${do_please}" -eq "Y") {
      ForEach (${service} in ${service_all}) {
        Write-Host "    Stopping `"$(${service}.Name)`":"
        Stop-Service -Force -Name ${service}.Name
        Start-Sleep 3
      }
    } Else {
      Read-Host "    Go ahead and stop your ${component} instances.`n      When the ${component} instances have completely stopped, press ENTER."
    }
    ${private:service_all} = (Get-Service -DisplayName "*${search}*" | Where-Object {$_.Status -eq "Running"})
  }
  Write-Host "    It appears no instances of ${component} are running. Proceeding ..."
}

Function Wait-DisableServices([string]${component}, [string]${search}) {
  ${private:service_all} = (Get-Service -DisplayName "*${search}*" | Where-Object {$_.StartType -ne "Disabled"})
  While (${service_all} -ne ${null}) {
    ForEach (${service} in ${service_all}) {
      Write-Host "    Disabling `"$(${service}.Name)`""
      Set-Service -Name ${service}.Name -StartupType Disabled
      Start-Sleep 3
    }
    ${private:service_all} = (Get-Service -DisplayName "*${search}*" | Where-Object {$_.StartType -ne "Disabled"})
  }
  Write-Host "    It appears no instances of ${component} are enabled. Proceeding ..."
}

Function Copy-OldFiles([string]${name}, [string]${source}, [string]${destination}, [string[]]$files) {
  ForEach (${file} in ${files}) {
    If ((${file} -ne ${null}) -and (${file} -ne "")) {
      If (Test-Path -Path ${source}\${file}) {
        If (Test-Path -Path ${destination}\${file}) {
          ${private:item} = (Get-Item ${destination}\${file})
          ${private:backup_name} = "$(${item}.BaseName)-backup$(${item}.Extension)"
          ${private:backup_full} = "$(${item}.Directory)\${backup_name}"
          Write-Host "    Backing up existing ${name} '${file}' to '${backup_name}'"
          Move-Item -Path ${item} -Destination ${backup_full} -Force
        }
        Write-Host "    Copying ${name} '${file}'"
        Copy-Item -Path "${source}\${file}" -Destination "$(Split-Path -Path "${destination}\${file}" -Parent)"
      }
    }
  }
}

Function Discard-SpecificFiles([string]${name}, [string]${destination}, [string[]]$files) {
  ForEach (${file} in ${files}) {
    If ((${file} -ne ${null}) -and (${file} -ne "")) {
      If (Test-Path ${destination}\${file}) {
        Write-Host "    Removing existing ${name} '${file}'"
        Remove-Item -Path "${destination}\${file}" -Recurse
      }
    }
  }
}

Function Disable-AJPConnector([string]${tomcat}) {
  Write-Host "    Disabling Tomcat AJP connector within `"${tomcat}\conf\server.xml`""

  Set-Content `
    -Path "${tomcat}\conf\server.xml" `
    -Value "$((Get-Content `"${tomcat}\conf\server.xml`" -Raw) -replace -join `
      (
        "(?ms)",
        "(\s+<!-- Define an AJP 1\.3 Connector on port [0-9]+ -->)(\s*)", # $1 $2
        "(<Connector port=``"[0-9]+``" protocol=``"AJP/1\.3``" redirectPort=``"[0-9]+``"\s*/>)(\s*)" # $3 $4
      ), -join `
      (
        "`$1`$2",
        "<!--`$2",
        "`$3`$2",
        "-->",
        "`$2`$2"
      )
    )"
}


Function Unzip([string]${zip}, [string]${destination})
{
  ${private:parent} = Split-Path -Path ${destination} -Parent
  ${private:leaf} = Split-Path -Path ${destination} -Leaf
  If (-not (Test-Path ${destination})) {
    ${null} = (New-Item -Path "${parent}" -Name "${leaf}" -ItemType "directory")
  }
  Write-Host "  Unpacking `"${zip}`" into `"${destination}`""
  [System.IO.Compression.ZipFile]::ExtractToDirectory(${zip}, ${destination})
  ${private:children} = @(Get-ChildItem -Path "${destination}")
  If (($(${children}.count) -eq 1) -and (${children}[0].Attributes -match "Directory")) {
    # When the only child-item is a directory we should strip off a layer, which should be safe
    #   Otherwise the (extracted) files in this folder are those that should stay
    # On some systems we cannot rename the extracted folder immediately
    ${private:rename_waiting} = ${TRUE}
    ${private:rename_count} = 0
    While (${private:rename_waiting}) {
      try {
        Rename-Item -Path "$(${children}[0].FullName)" -newName "${leaf}" -ErrorAction Stop
        ${private:rename_waiting} = ${FALSE}
      } catch [System.IO.IOException] {
        If (${private:rename_count}++ -ge 5) {
          # We have waited long enough; report the error
          ${private:rename_waiting} = ${FALSE}
          throw $_
        } Else {
          Start-Sleep -Seconds 2
        }
      }
    }
    Get-ChildItem -Path "${destination}\${leaf}" -Recurse | Move-Item -Destination "${destination}"
    Remove-Item -Path "${destination}\${leaf}"
  }
}

Function Is-Cloud([string]${server}) {
  return ((${server} -like "*.postgres.database.azure.com") -or (${server} -like "*.rds.amazonaws.com"))
}

Function Backup-Databases(
    [string]${admin},
    [string]${server},
    [string]${port},
    [string]${postgresql_path},
    [string]${destination_files},
    [string[]]${databases}
) {
  If (-not (Test-Path ${destination_files})) {
    ${null} = (New-Item -Path "${destination_files}" -ItemType "directory")
  }

  ForEach (${db} in ${databases}) {
    ${private:file} = "${destination_files}\${db}.dump"
    Write-Host "    Backing up `"${db}`" to `"${file}`""
    Write-Host (
    "    ",
    "Start-Process",
      "-FilePath ${postgresql_path}\bin\pg_dump.exe",
      "-Wait",
      "-NoNewWindow",
      "-ArgumentList",
        "`"-h ${server}`",",
        "`"-p ${port}`",",
        "`"-U ${admin}`",",
        "`"-Fc`",",
        "`"-f ${file}`",",
        "`"${db}`""
    )
    ${PROCESS_START_INFO} = New-Object System.Diagnostics.ProcessStartInfo
    ${PROCESS_START_INFO}.RedirectStandardError = ${TRUE}
    ${PROCESS_START_INFO}.UseShellExecute = ${FALSE}
    ${PROCESS_START_INFO}.FileName = "${postgresql_path}\bin\pg_dump.exe"
    ${PROCESS_START_INFO}.Arguments = `
      "-h ${server}", `
      "-p ${port}", `
      "-U ${admin}", `
      "-Fc", `
      "-f ${file}", `
      "${db}"
    ${PROCESS_RUN} = New-Object System.Diagnostics.Process
    ${PROCESS_RUN}.StartInfo = ${PROCESS_START_INFO}
    ${PROCESS_RUN}.Start() | Out-Null
    ${PROCESS_RUN}.WaitForExit()
    Write-Host "$( ${PROCESS_RUN}.StandardError.ReadToEnd() `
      -replace -join ( `"(?ms)`",
        "pg_dump: \[archiver \(db\)\] connection to database ``"${db}``" failed: FATAL:  database ``"${db}``" does not exist"
      ), "    pg_dump: [archiver (db)] connection to database ``"${db}``" failed: ``"${db}.dump``" will safely remain empty"
    )"
  }

  ${private:roles} = "${destination_files}\roles.sql"
  Write-Host "    Backing up roles to `"${roles}`""
  ${private:cloud} = $(Is-Cloud ${server})
  If (${cloud}) {
    ${private:extra_params} = "--no-role-passwords"
  } Else {
    ${private:extra_params} = " "
  }
  Write-Host (
    "    ",
    "Start-Process",
      "-FilePath ${postgresql_path}\bin\pg_dumpall.exe",
      "-Wait",
      "-NoNewWindow",
      "-ArgumentList",
        "`"-h ${server}`",",
        "`"-p ${port}`",",
        "`"-U ${admin}`",",
        "`"-f ${roles}`",",
        "`"--roles-only`",",
        "`"${extra_params}`""
  )
  ${PROCESS_START_INFO} = New-Object System.Diagnostics.ProcessStartInfo
  ${PROCESS_START_INFO}.RedirectStandardError = ${TRUE}
  ${PROCESS_START_INFO}.UseShellExecute = ${FALSE}
  ${PROCESS_START_INFO}.FileName = "${postgresql_path}\bin\pg_dumpall.exe"
  ${PROCESS_START_INFO}.Arguments = `
    "-h ${server}", `
    "-p ${port}", `
    "-U ${admin}", `
    "-f ${roles}", `
    "--roles-only", `
    "${extra_params}"
  ${PROCESS_RUN} = New-Object System.Diagnostics.Process
  ${PROCESS_RUN}.StartInfo = ${PROCESS_START_INFO}
  ${PROCESS_RUN}.Start() | Out-Null
  ${PROCESS_RUN}.WaitForExit()
  Write-Host "$(${PROCESS_RUN}.StandardError.ReadToEnd() `
  )"
}

Function Restore-Databases(
    [string]${admin},
    [string]${server},
    [string]${port},
    [string]${postgresql_path},
    [string]${destination_files},
    [string[]]${databases}
) {
  If (-not (Test-Path ${destination_files})) { return }

  ${private:roles} = "${destination_files}\roles.sql"
  Write-Host "    Restoring roles from `"${roles}`""
  Write-Host (
    "    ",
    "Start-Process",
      "-FilePath ${postgresql_path}\bin\psql.exe",
      "-Wait",
      "-NoNewWindow",
      "-ArgumentList",
        "`"-h ${server}`",",
        "`"-p ${port}`",",
        "`"-U ${admin}`",",
        "`"-d postgres`",",
        "`"-f ${roles}`""
  )
  ${PROCESS_START_INFO} = New-Object System.Diagnostics.ProcessStartInfo
  ${PROCESS_START_INFO}.RedirectStandardError = ${TRUE}
  ${PROCESS_START_INFO}.UseShellExecute = ${FALSE}
  ${PROCESS_START_INFO}.FileName = "${postgresql_path}\bin\psql.exe"
  ${PROCESS_START_INFO}.Arguments = `
    "-h ${server}", `
    "-p ${port}", `
    "-U ${admin}", `
    "-d postgres", `
    "-f ${roles}"
  ${PROCESS_RUN} = New-Object System.Diagnostics.Process
  ${PROCESS_RUN}.StartInfo = ${PROCESS_START_INFO}
  ${PROCESS_RUN}.Start() | Out-Null
  ${PROCESS_RUN}.WaitForExit()
  Write-Host "$( ${PROCESS_RUN}.StandardError.ReadToEnd() `
    -replace -join ( `"(?ms)`",
      "ERROR:  role ``"(.*?)``" already exists"
    ), "Redundant query:  role ``"`$1``" already exists" `
    -replace -join ( `"(?ms)`",
      "NOTICE:  role ``"(.*?)``" is already a member of role ``"(.*?)``""
    ), "Redundant assignment:  role ``"`$1``" is already a member of role ``"`$2``""
  )"

  ForEach (${db} in ${databases}) {
    ${private:file} = "${destination_files}\${db}.dump"
    If ((Get-Content "${private:file}") -ne ${NULL}) {
      Write-Host "    Restoring `"${db}`" from `"${file}`""
      # Delete the DB if it exists, which it should not
      Write-Host (
        "    ",
        "Start-Process",
          "-FilePath ${postgresql_path}\bin\dropdb.exe",
          "-Wait",
          "-NoNewWindow",
          "-ArgumentList",
            "`"-h ${server}`",",
            "`"-p ${port}`",",
            "`"-U ${admin}`",",
            "`"${db}`""
      )
      ${PROCESS_START_INFO} = New-Object System.Diagnostics.ProcessStartInfo
      ${PROCESS_START_INFO}.RedirectStandardError = ${TRUE}
      ${PROCESS_START_INFO}.UseShellExecute = ${FALSE}
      ${PROCESS_START_INFO}.FileName = "${postgresql_path}\bin\dropdb.exe"
      ${PROCESS_START_INFO}.Arguments = `
        "-h ${server}", `
        "-p ${port}", `
        "-U ${admin}", `
        "${db}"
      ${PROCESS_RUN} = New-Object System.Diagnostics.Process
      ${PROCESS_RUN}.StartInfo = ${PROCESS_START_INFO}
      ${PROCESS_RUN}.Start() | Out-Null
      ${PROCESS_RUN}.WaitForExit()
      Write-Host "$( ${PROCESS_RUN}.StandardError.ReadToEnd() `
        -replace -join ( `"(?ms)`",
          "dropdb: database removal failed: ERROR:  database ``"${db}``" does not exist"
        ), "    dropdb: database removal redundant:  database ``"${db}``" does not exist"
      )"

      # Create the DB
      Write-Host (
        "    ",
        "Start-Process",
          "-FilePath ${postgresql_path}\bin\createdb.exe",
          "-Wait",
          "-NoNewWindow",
          "-ArgumentList",
            "`"-h ${server}`",",
            "`"-p ${port}`",",
            "`"-U ${admin}`",",
            "`"-T template0`",",
            "`"${db}`""
      )
      ${PROCESS_START_INFO} = New-Object System.Diagnostics.ProcessStartInfo
      ${PROCESS_START_INFO}.RedirectStandardError = ${TRUE}
      ${PROCESS_START_INFO}.UseShellExecute = ${FALSE}
      ${PROCESS_START_INFO}.FileName = "${postgresql_path}\bin\createdb.exe"
      ${PROCESS_START_INFO}.Arguments = `
        "-h ${server}", `
        "-p ${port}", `
        "-U ${admin}", `
        "-T template0", `
        "${db}"
      ${PROCESS_RUN} = New-Object System.Diagnostics.Process
      ${PROCESS_RUN}.StartInfo = ${PROCESS_START_INFO}
      ${PROCESS_RUN}.Start() | Out-Null
      ${PROCESS_RUN}.WaitForExit()
      Write-Host "$( ${PROCESS_RUN}.StandardError.ReadToEnd() `
        -replace -join ( `"(?ms)`",
          "createdb: database creation failed: ERROR:  database ``"${db}``" already exists"
        ), "    createdb: database creation redundant:  database ``"${db}``" already exists"
      )"

      # Populate the database with the backed-up data
      Write-Host (
        "    ",
        "Start-Process",
          "-FilePath ${postgresql_path}\bin\pg_restore.exe",
          "-Wait",
          "-NoNewWindow",
          "-ArgumentList",
            "`"-h ${server}`",",
            "`"-p ${port}`",",
            "`"-U ${admin}`",",
            "`"-d ${db}`",",
            "`"-Fc ${file}`""
      )
      ${PROCESS_START_INFO} = New-Object System.Diagnostics.ProcessStartInfo
      ${PROCESS_START_INFO}.RedirectStandardError = ${TRUE}
      ${PROCESS_START_INFO}.UseShellExecute = ${FALSE}
      ${PROCESS_START_INFO}.FileName = "${postgresql_path}\bin\pg_restore.exe"
      ${PROCESS_START_INFO}.Arguments = `
        "-h ${server}",
        "-p ${port}",
        "-U ${admin}",
        "-d ${db}",
        "-Fc ${file}"
      ${PROCESS_RUN} = New-Object System.Diagnostics.Process
      ${PROCESS_RUN}.StartInfo = ${PROCESS_START_INFO}
      ${PROCESS_RUN}.Start() | Out-Null
      ${PROCESS_RUN}.WaitForExit()
      Write-Host "$(${PROCESS_RUN}.StandardError.ReadToEnd() `
        -replace -join ( `"(?ms)`",
          `"(pg_restore: \[archiver \(db\)\] )(Error)( while PROCESSING TOC:`r`n)`",
          `"(pg_restore: \[archiver \(db\)\] )(Error)( from TOC entry 3; 2615 2200 SCHEMA public postgres`r`n)`",
          `"(pg_restore: \[archiver \(db\)\] )(could not execute query: ERROR)(:  schema ``"public``" already exists)`"
        ), `"    `$1Redundancy`$3    `$4Redundancy`$6    `$7Skipping query execution`$9`" `
        -replace -join ( `"(?ms)`",
          "WARNING: errors ignored on restore: [0-9]+"
        ), `"`" `
        -replace -join ( `"(?ms)`",
          "(pg_restore: \[custom archiver\] )(could not read from)( input file: end of file)"
        ), `"    `$1ignoring empty input file ``"${db}.dump``"`"
      )"
    } Else {
      Write-Host "    Ignoring empty file `"${db}.dump`"`n"
    }
  }
}

Function Show-Differences(
    [string]${reference},
    [string]${difference}
) {
  ${do_please} = Read-Host "  Do you want to diff/compare `"${reference}`" with `"${difference}`"?`n    It can potentially take several minutes. [Y/n]"
  If ("${do_please}" -eq "" -Or "${do_please}" -eq "y" -Or "${do_please}" -eq "Y") {
    ${comparisons} = @{
      ReferenceObject = (Get-ChildItem -Recurse -Path ${reference})
      DifferenceObject = (Get-ChildItem -Recurse -Path ${difference})
    }

    Compare-Object @comparisons -PassThru
  }
}

Function Get-ResolvedPath(
    [string]${path}
) {
  ${item} = (Get-Item -Path ${path})
  If (${item}.Attributes -match "ReparsePoint") {
    ${local:path_target} = ${item}.Target
    If ([System.IO.Path]::IsPathRooted(${local:path_target})) {
      ${response} = "${local:path_target}"
    } Else {
      ${response} = "$(Split-Path -Path "$(${item}.FullName)" -Parent)\${local:path_target}"
    }
  } Else {
    ${response} = "$(${item}.FullName)"
  }
  return ${response}
}

Function Summary-Print() {
  Write-Host "`nSummary"
  ForEach (${local:register} in ${global:SUMMARY_REGISTRATION}) {
    If ( "${register}" -eq "activemq" ) { Summary-PrintOne "ActiveMQ" ${active_mq_summary_code} }
    If ( "${register}" -eq "java" ) { Summary-PrintOne "Java" ${java_summary_code} }
    If ( "${register}" -eq "postgresql" ) { Summary-PrintOne "PostgreSQL" ${postgresql_summary_code} }
    If ( "${register}" -eq "tomcat" ) { Summary-PrintOne "Tomcat" ${tomcat_summary_code} }
  }
}

Function Summary-PrintOne(
  [string]${name},
  [int]${code}
) {
  Switch(${code}) {
    ${SUMMARY_CODE_COMPLETED} { Write-Host "  ${name} updated successfully" }
    ${SUMMARY_CODE_SKIPPED} { Write-Host "  ${name} update was possible but skipped" }
    ${SUMMARY_CODE_MISSED} { Write-Host "  ${name} update unavailable due to missing installation file" }
    ${SUMMARY_CODE_FAILED} { Write-Host "  ${name} failed to update" }
  }
}

Function Summary-Register(
  [string[]]${components}
) {
  ForEach (${local:component} in ${components}) {
    ${private:can_add} = ${TRUE}
    ForEach (${local:register} in ${global:SUMMARY_REGISTRATION}) {
      If ("${register}" -eq "${component}") {
        ${private:can_add} = ${FALSE}
        continue
      }
    }
    If (${can_add}) {
      ${global:SUMMARY_REGISTRATION} = @(
        ${global:SUMMARY_REGISTRATION}
        ${component}
      )
    }
  }
}


# SIG # Begin signature block
# MIIggQYJKoZIhvcNAQcCoIIgcjCCIG4CAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCCDUUgKknrFlVNc
# Twedq72O8VWhVFnJuyydAzyxpmLCJqCCG4cwggPFMIICraADAgECAhACrFwmagtA
# m48LefKuRiV3MA0GCSqGSIb3DQEBBQUAMGwxCzAJBgNVBAYTAlVTMRUwEwYDVQQK
# EwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xKzApBgNV
# BAMTIkRpZ2lDZXJ0IEhpZ2ggQXNzdXJhbmNlIEVWIFJvb3QgQ0EwHhcNMDYxMTEw
# MDAwMDAwWhcNMzExMTEwMDAwMDAwWjBsMQswCQYDVQQGEwJVUzEVMBMGA1UEChMM
# RGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29tMSswKQYDVQQD
# EyJEaWdpQ2VydCBIaWdoIEFzc3VyYW5jZSBFViBSb290IENBMIIBIjANBgkqhkiG
# 9w0BAQEFAAOCAQ8AMIIBCgKCAQEAxszlc+b71LvlLS0ypt/lgT/JzSVJtnEqw9WU
# NGeiChywX2mmQLHEt7KP0JikqUFZOtPclNY823Q4pErMTSWC90qlUxI47vNJbXGR
# fmO2q6Zfw6SE+E9iUb74xezbOJLjBuUIkQzEKEFV+8taiRV+ceg1v01yCT2+OjhQ
# W3cxG42zxyRFmqesbQAUWgS3uhPrUQqYQUEiTmVhh4FBUKZ5XIneGUpX1S7mXRxT
# LH6YzRoGFqRoc9A0BBNcoXHTWnxV215k4TeHMFYE5RG0KYAS8Xk5iKICEXwnZreI
# t3jyygqoOKsKZMK/Zl2VhMGhJR6HXRpQCyASzEG7bgtROLhLywIDAQABo2MwYTAO
# BgNVHQ8BAf8EBAMCAYYwDwYDVR0TAQH/BAUwAwEB/zAdBgNVHQ4EFgQUsT7DaQP4
# v0cB1JgmGggC72NkK8MwHwYDVR0jBBgwFoAUsT7DaQP4v0cB1JgmGggC72NkK8Mw
# DQYJKoZIhvcNAQEFBQADggEBABwaBpfc15yfPIhmBghXIdshR/gqZ6q/GDJ2QBBX
# wYrzetkRZY41+p78RbWe2UwxS7iR6EMsjrN4ztvjU3lx1uUhlAHaVYeaJGT2imbM
# 3pw3zag0sWmbI8ieeCIrcEPjVUcxYRnvWMWFL04w9qAxFiPI5+JlFjPLvxoboD34
# yl6LMYtgCIktDAZcUrfE+QqY0RVfnxK+fDZjOL1EpH/kJisKxJdpDemM4sAQV7jI
# dhKRVfJIadi8KgJbD0TUIDHb9LpwJl2QYJ68SxcJL7TLHkNoyQcnwdJc9+ohuWgS
# nDycv578gFybY83sR6olJ2egN/MAgn1U16n46S4To3foH0owggUoMIIEEKADAgEC
# AhANNvCcqoPZqIxHb9ZBXrLvMA0GCSqGSIb3DQEBCwUAMHYxCzAJBgNVBAYTAlVT
# MRUwEwYDVQQKEwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5j
# b20xNTAzBgNVBAMTLERpZ2lDZXJ0IFNIQTIgSGlnaCBBc3N1cmFuY2UgQ29kZSBT
# aWduaW5nIENBMB4XDTE4MTEzMDAwMDAwMFoXDTIxMTIwODEyMDAwMFowZzELMAkG
# A1UEBhMCVVMxDjAMBgNVBAgTBVRleGFzMRAwDgYDVQQHEwdIb3VzdG9uMRowGAYD
# VQQKExFOZXRJUSBDb3Jwb3JhdGlvbjEaMBgGA1UEAxMRTmV0SVEgQ29ycG9yYXRp
# b24wggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQCeZBYzhJbJO6boi5Tm
# q+1hqQvSRI1lsJZx5CqI3uxWjgFpDyaGGaNeXCAYAcw/YXPNt+wyDMiRqWQ2bz33
# b/EkLDlNZaJUeJVSzG/JBlm8Wtgu+veqi9bph+pBR9cKOYz6cQG3GxD4SPU/zIhD
# wwt/M963pOaVkt4kl56jWWPjRQTnb71KWsyxP/c/Z5WsbSH+QOqur/JayO4LkH/i
# yHT44R+Xb5OK9/XAh3eFmYbFxgwudwx4kkpiS6ckneCzRE+SN7JyvdTS+lThopN8
# 5K41BgY2Edc+k0628AiSo/xuPpzimOn+Hxouax9MfXShWT6Hh+QNNFI/2h7QTS6Y
# t3uBAgMBAAGjggG/MIIBuzAfBgNVHSMEGDAWgBRnnQ8gCQzMijrlgkZyYvzxzJDl
# QDAdBgNVHQ4EFgQU4oPg/2aI0v0mcRoEwVgbZEHhuRYwDgYDVR0PAQH/BAQDAgeA
# MBMGA1UdJQQMMAoGCCsGAQUFBwMDMG0GA1UdHwRmMGQwMKAuoCyGKmh0dHA6Ly9j
# cmwzLmRpZ2ljZXJ0LmNvbS9zaGEyLWhhLWNzLWcxLmNybDAwoC6gLIYqaHR0cDov
# L2NybDQuZGlnaWNlcnQuY29tL3NoYTItaGEtY3MtZzEuY3JsMEwGA1UdIARFMEMw
# NwYJYIZIAYb9bAMLMCowKAYIKwYBBQUHAgEWHGh0dHBzOi8vd3d3LmRpZ2ljZXJ0
# LmNvbS9DUFMwCAYGZ4EMAQQBMIGIBggrBgEFBQcBAQR8MHowJAYIKwYBBQUHMAGG
# GGh0dHA6Ly9vY3NwLmRpZ2ljZXJ0LmNvbTBSBggrBgEFBQcwAoZGaHR0cDovL2Nh
# Y2VydHMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0U0hBMkhpZ2hBc3N1cmFuY2VDb2Rl
# U2lnbmluZ0NBLmNydDAMBgNVHRMBAf8EAjAAMA0GCSqGSIb3DQEBCwUAA4IBAQBz
# dAg+QLbVAoX48OX0aZWGxUxBt/VCvwfWvyo0Z81E32M97dPS+z8rDPOQcD5cFmI/
# +GO3VAENqbAYuMN+OMq8qTqXU3EnmJekiF4rqIBSRBDUyYTrDvctyp/uM0DlmXcy
# sXEMEIa4t3UzEu0Ul/GAovRZVJ0ljQEAPsUP0JihcO7vakhfmP7B6AvKhYIUIJIR
# av7ML1aizJcKeV8lj1gDuSrG3KXqxGKnGSJnT1930YVJi8dmMvbx1AUmrEpiIQ6H
# LkKf1xAexMzAm/tpFp7W6vNfhST/UqkaJqOoR8FuhMmrCcCqjgKvlMjmMJaYkQ0X
# X/4fpBGqjWkNrFxnIuCAMIIFTzCCBDegAwIBAgIQC34QkDw4SQ/6L2eah6GnuTAN
# BgkqhkiG9w0BAQsFADBsMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQg
# SW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29tMSswKQYDVQQDEyJEaWdpQ2Vy
# dCBIaWdoIEFzc3VyYW5jZSBFViBSb290IENBMB4XDTEzMTAyMjEyMDAwMFoXDTI4
# MTAyMjEyMDAwMFowdjELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IElu
# YzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTE1MDMGA1UEAxMsRGlnaUNlcnQg
# U0hBMiBIaWdoIEFzc3VyYW5jZSBDb2RlIFNpZ25pbmcgQ0EwggEiMA0GCSqGSIb3
# DQEBAQUAA4IBDwAwggEKAoIBAQC0Sl59Bw9B3sT1dhY2vXH/zz9Pc0uc0Q3+SstX
# WF6FFt0CFVSZ8I88L00CeBBoyNg1Sz/B92fOmByuM7ktHaQKVJPEhaLfNbH18Tyn
# szT7XUjJRsliRLxImesoSVPDPY/ADt41mOliUd89a0Bh7gRB2s+nXFaW0flMt0SE
# h5hp5YK5E+ZVv8iScJIKMW9/izKrz2tan2LEPu6+7VmkU38L8VKIinsKZyTLkM3s
# 0k00TLDhtZ+cxvZvLM3mylN0AZ9nNd44SS3O7TlEghl5Thqytfu7ePBJZqfP+lyW
# dZKLGnLZ/1CSU8w+wkMyCRqGE2k8+4EyMzJkdXMoJh0IMDsHAgMBAAGjggHhMIIB
# 3TASBgNVHRMBAf8ECDAGAQH/AgEAMA4GA1UdDwEB/wQEAwIBhjATBgNVHSUEDDAK
# BggrBgEFBQcDAzB/BggrBgEFBQcBAQRzMHEwJAYIKwYBBQUHMAGGGGh0dHA6Ly9v
# Y3NwLmRpZ2ljZXJ0LmNvbTBJBggrBgEFBQcwAoY9aHR0cDovL2NhY2VydHMuZGln
# aWNlcnQuY29tL0RpZ2lDZXJ0SGlnaEFzc3VyYW5jZUVWUm9vdENBLmNydDCBjwYD
# VR0fBIGHMIGEMECgPqA8hjpodHRwOi8vY3JsNC5kaWdpY2VydC5jb20vRGlnaUNl
# cnRIaWdoQXNzdXJhbmNlRVZSb290Q0EuY3JsMECgPqA8hjpodHRwOi8vY3JsMy5k
# aWdpY2VydC5jb20vRGlnaUNlcnRIaWdoQXNzdXJhbmNlRVZSb290Q0EuY3JsME8G
# A1UdIARIMEYwOAYKYIZIAYb9bAACBDAqMCgGCCsGAQUFBwIBFhxodHRwczovL3d3
# dy5kaWdpY2VydC5jb20vQ1BTMAoGCGCGSAGG/WwDMB0GA1UdDgQWBBRnnQ8gCQzM
# ijrlgkZyYvzxzJDlQDAfBgNVHSMEGDAWgBSxPsNpA/i/RwHUmCYaCALvY2QrwzAN
# BgkqhkiG9w0BAQsFAAOCAQEAag7/fhN8BqVLwC6M+VNkCeK6WJEwUOzMn+HTqC9I
# RjYYKdB4KF+YVkAPHrq9sTuHXNxb2CAN7RoWTdURJCFL8SdpkBPrEaEB2v21TnlZ
# db04KmrD9o5BK4qii9csUVHZnKDI4066bKhH0k7RaB+MAlc7sylqjmogKrnyAGJk
# usjpAPnMpNS6mjXYryxlbBZ8WCHeSjDQ+uskXQbJnRa3rUpF0yXiDPBAqlxNrH7N
# BoK5dkZpCNgytoL+46lYNEMbjmdnlz9oMRY2OJU+h/fHw6+dencZ2d6Ttf1uK/yU
# +T23TBI1LDC+6I2eBXCaSBP0jNbnHqw456jzrQy3euxn7TCCBmowggVSoAMCAQIC
# EAMBmgI6/1ixa9bV6uYX8GYwDQYJKoZIhvcNAQEFBQAwYjELMAkGA1UEBhMCVVMx
# FTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNv
# bTEhMB8GA1UEAxMYRGlnaUNlcnQgQXNzdXJlZCBJRCBDQS0xMB4XDTE0MTAyMjAw
# MDAwMFoXDTI0MTAyMjAwMDAwMFowRzELMAkGA1UEBhMCVVMxETAPBgNVBAoTCERp
# Z2lDZXJ0MSUwIwYDVQQDExxEaWdpQ2VydCBUaW1lc3RhbXAgUmVzcG9uZGVyMIIB
# IjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAo2Rd/Hyz4II14OD2xirmSXU7
# zG7gU6mfH2RZ5nxrf2uMnVX4kuOe1VpjWwJJUNmDzm9m7t3LhelfpfnUh3SIRDsZ
# yeX1kZ/GFDmsJOqoSyyRicxeKPRktlC39RKzc5YKZ6O+YZ+u8/0SeHUOplsU/UUj
# joZEVX0YhgWMVYd5SEb3yg6Np95OX+Koti1ZAmGIYXIYaLm4fO7m5zQvMXeBMB+7
# NgGN7yfj95rwTDFkjePr+hmHqH7P7IwMNlt6wXq4eMfJBi5GEMiN6ARg27xzdPpO
# 2P6qQPGyznBGg+naQKFZOtkVCVeZVjCT88lhzNAIzGvsYkKRrALA76TwiRGPdwID
# AQABo4IDNTCCAzEwDgYDVR0PAQH/BAQDAgeAMAwGA1UdEwEB/wQCMAAwFgYDVR0l
# AQH/BAwwCgYIKwYBBQUHAwgwggG/BgNVHSAEggG2MIIBsjCCAaEGCWCGSAGG/WwH
# ATCCAZIwKAYIKwYBBQUHAgEWHGh0dHBzOi8vd3d3LmRpZ2ljZXJ0LmNvbS9DUFMw
# ggFkBggrBgEFBQcCAjCCAVYeggFSAEEAbgB5ACAAdQBzAGUAIABvAGYAIAB0AGgA
# aQBzACAAQwBlAHIAdABpAGYAaQBjAGEAdABlACAAYwBvAG4AcwB0AGkAdAB1AHQA
# ZQBzACAAYQBjAGMAZQBwAHQAYQBuAGMAZQAgAG8AZgAgAHQAaABlACAARABpAGcA
# aQBDAGUAcgB0ACAAQwBQAC8AQwBQAFMAIABhAG4AZAAgAHQAaABlACAAUgBlAGwA
# eQBpAG4AZwAgAFAAYQByAHQAeQAgAEEAZwByAGUAZQBtAGUAbgB0ACAAdwBoAGkA
# YwBoACAAbABpAG0AaQB0ACAAbABpAGEAYgBpAGwAaQB0AHkAIABhAG4AZAAgAGEA
# cgBlACAAaQBuAGMAbwByAHAAbwByAGEAdABlAGQAIABoAGUAcgBlAGkAbgAgAGIA
# eQAgAHIAZQBmAGUAcgBlAG4AYwBlAC4wCwYJYIZIAYb9bAMVMB8GA1UdIwQYMBaA
# FBUAEisTmLKZB+0e36K+Vw0rZwLNMB0GA1UdDgQWBBRhWk0ktkkynUoqeRqDS/Qe
# icHKfTB9BgNVHR8EdjB0MDigNqA0hjJodHRwOi8vY3JsMy5kaWdpY2VydC5jb20v
# RGlnaUNlcnRBc3N1cmVkSURDQS0xLmNybDA4oDagNIYyaHR0cDovL2NybDQuZGln
# aWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEQ0EtMS5jcmwwdwYIKwYBBQUHAQEE
# azBpMCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2VydC5jb20wQQYIKwYB
# BQUHMAKGNWh0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3Vy
# ZWRJRENBLTEuY3J0MA0GCSqGSIb3DQEBBQUAA4IBAQCdJX4bM02yJoFcm4bOIyAP
# gIfliP//sdRqLDHtOhcZcRfNqRu8WhY5AJ3jbITkWkD73gYBjDf6m7GdJH7+IKRX
# rVu3mrBgJuppVyFdNC8fcbCDlBkFazWQEKB7l8f2P+fiEUGmvWLZ8Cc9OB0obzpS
# CfDscGLTYkuw4HOmksDTjjHYL+NtFxMG7uQDthSr849Dp3GdId0UyhVdkkHa+Q+B
# 0Zl0DSbEDn8btfWg8cZ3BigV6diT5VUW8LsKqxzbXEgnZsijiwoc5ZXarsQuWaBh
# 3drzbaJh6YoLbewSGL33VVRAA5Ira8JRwgpIr7DUbuD0FAo6G+OPPcqvao173NhE
# MIIGzTCCBbWgAwIBAgIQBv35A5YDreoACus/J7u6GzANBgkqhkiG9w0BAQUFADBl
# MQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3
# d3cuZGlnaWNlcnQuY29tMSQwIgYDVQQDExtEaWdpQ2VydCBBc3N1cmVkIElEIFJv
# b3QgQ0EwHhcNMDYxMTEwMDAwMDAwWhcNMjExMTEwMDAwMDAwWjBiMQswCQYDVQQG
# EwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNl
# cnQuY29tMSEwHwYDVQQDExhEaWdpQ2VydCBBc3N1cmVkIElEIENBLTEwggEiMA0G
# CSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQDogi2Z+crCQpWlgHNAcNKeVlRcqcTS
# QQaPyTP8TUWRXIGf7Syc+BZZ3561JBXCmLm0d0ncicQK2q/LXmvtrbBxMevPOkAM
# Rk2T7It6NggDqww0/hhJgv7HxzFIgHweog+SDlDJxofrNj/YMMP/pvf7os1vcyP+
# rFYFkPAyIRaJxnCI+QWXfaPHQ90C6Ds97bFBo+0/vtuVSMTuHrPyvAwrmdDGXRJC
# geGDboJzPyZLFJCuWWYKxI2+0s4Grq2Eb0iEm09AufFM8q+Y+/bOQF1c9qjxL6/s
# iSLyaxhlscFzrdfx2M8eCnRcQrhofrfVdwonVnwPYqQ/MhRglf0HBKIJAgMBAAGj
# ggN6MIIDdjAOBgNVHQ8BAf8EBAMCAYYwOwYDVR0lBDQwMgYIKwYBBQUHAwEGCCsG
# AQUFBwMCBggrBgEFBQcDAwYIKwYBBQUHAwQGCCsGAQUFBwMIMIIB0gYDVR0gBIIB
# yTCCAcUwggG0BgpghkgBhv1sAAEEMIIBpDA6BggrBgEFBQcCARYuaHR0cDovL3d3
# dy5kaWdpY2VydC5jb20vc3NsLWNwcy1yZXBvc2l0b3J5Lmh0bTCCAWQGCCsGAQUF
# BwICMIIBVh6CAVIAQQBuAHkAIAB1AHMAZQAgAG8AZgAgAHQAaABpAHMAIABDAGUA
# cgB0AGkAZgBpAGMAYQB0AGUAIABjAG8AbgBzAHQAaQB0AHUAdABlAHMAIABhAGMA
# YwBlAHAAdABhAG4AYwBlACAAbwBmACAAdABoAGUAIABEAGkAZwBpAEMAZQByAHQA
# IABDAFAALwBDAFAAUwAgAGEAbgBkACAAdABoAGUAIABSAGUAbAB5AGkAbgBnACAA
# UABhAHIAdAB5ACAAQQBnAHIAZQBlAG0AZQBuAHQAIAB3AGgAaQBjAGgAIABsAGkA
# bQBpAHQAIABsAGkAYQBiAGkAbABpAHQAeQAgAGEAbgBkACAAYQByAGUAIABpAG4A
# YwBvAHIAcABvAHIAYQB0AGUAZAAgAGgAZQByAGUAaQBuACAAYgB5ACAAcgBlAGYA
# ZQByAGUAbgBjAGUALjALBglghkgBhv1sAxUwEgYDVR0TAQH/BAgwBgEB/wIBADB5
# BggrBgEFBQcBAQRtMGswJAYIKwYBBQUHMAGGGGh0dHA6Ly9vY3NwLmRpZ2ljZXJ0
# LmNvbTBDBggrBgEFBQcwAoY3aHR0cDovL2NhY2VydHMuZGlnaWNlcnQuY29tL0Rp
# Z2lDZXJ0QXNzdXJlZElEUm9vdENBLmNydDCBgQYDVR0fBHoweDA6oDigNoY0aHR0
# cDovL2NybDMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9vdENBLmNy
# bDA6oDigNoY0aHR0cDovL2NybDQuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJl
# ZElEUm9vdENBLmNybDAdBgNVHQ4EFgQUFQASKxOYspkH7R7for5XDStnAs0wHwYD
# VR0jBBgwFoAUReuir/SSy4IxLVGLp6chnfNtyA8wDQYJKoZIhvcNAQEFBQADggEB
# AEZQPsm3KCSnOB22WymvUs9S6TFHq1Zce9UNC0Gz7+x1H3Q48rJcYaKclcNQ5IK5
# I9G6OoZyrTh4rHVdFxc0ckeFlFbR67s2hHfMJKXzBBlVqefj56tizfuLLZDCwNK1
# lL1eT7EF0g49GqkUW6aGMWKoqDPkmzmnxPXOHXh2lCVz5Cqrz5x2S+1fwksW5Etw
# TACJHvzFebxMElf+X+EevAJdqP77BzhPDcZdkbkPZ0XN1oPt55INjbFpjE/7WeAj
# D9KqrgB87pxCDs+R1ye3Fu4Pw718CqDuLAhVhSK46xgaTfwqIa1JMYNHlXdx3LEb
# S0scEJx3FMGdTy9alQgpECYxggRQMIIETAIBATCBijB2MQswCQYDVQQGEwJVUzEV
# MBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29t
# MTUwMwYDVQQDEyxEaWdpQ2VydCBTSEEyIEhpZ2ggQXNzdXJhbmNlIENvZGUgU2ln
# bmluZyBDQQIQDTbwnKqD2aiMR2/WQV6y7zANBglghkgBZQMEAgEFAKCBhDAYBgor
# BgEEAYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJAzEMBgorBgEEAYI3AgEE
# MBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEVMC8GCSqGSIb3DQEJBDEiBCDn
# gJRZu8QBsmBFnJeVM/1t+ISqPez8SjiNSrh05H4aNjANBgkqhkiG9w0BAQEFAASC
# AQA+3t+kYcPoCc/AxQlvvVHgbAVABVcyAkhD1qy4Ve2AVO4vjp3Hw07E7EKaPjbR
# CkRvqRCOvvOP0OVPezCzp+i++cTQMvhTyqCi+rTDf3sSYYRTRDcHa7RDga/qSMug
# +9iXDuNW2t6rOYdHBinLOsW/q3XJ1bw2OlY3EfA5XdZ3Y215yG9KlC3KanhIe/wU
# n6TNSNZSuMx1w+9lEwD1WK9h3ECu1DVPv7f7Hj4t0GH7hVojNOzzKqR+NofXRZPs
# BW/CmTLmWEa6GelM4oORESTsIlhDrz1eelAblVXaH8tnVDsrYz7hvcHO38vQbYOi
# rSy3ZMQ1w1oqzxB9UgoMVZK8oYICDzCCAgsGCSqGSIb3DQEJBjGCAfwwggH4AgEB
# MHYwYjELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UE
# CxMQd3d3LmRpZ2ljZXJ0LmNvbTEhMB8GA1UEAxMYRGlnaUNlcnQgQXNzdXJlZCBJ
# RCBDQS0xAhADAZoCOv9YsWvW1ermF/BmMAkGBSsOAwIaBQCgXTAYBgkqhkiG9w0B
# CQMxCwYJKoZIhvcNAQcBMBwGCSqGSIb3DQEJBTEPFw0yMDA0MjEwMTA4MDdaMCMG
# CSqGSIb3DQEJBDEWBBRL7WW3/i7YiXVpEvh1Qqd7kfBW7TANBgkqhkiG9w0BAQEF
# AASCAQCOLSwp9kjtphNlIur94BmSJ/QCE9XZnwfQNuM1TGeaGWNl3jqI397HuESQ
# K8vU87TYYecxrzavUvaZnBTYHmP+jRtSGC57xKXWSjTmI2zti/hJGuvx6EAAkXI3
# 1cgJFQgtN0LkojS+BrryTWgxTZbGiqHPpYCHEP7jBO4wKQtewE9BgzuAM0KF1FHJ
# H8ntRcMcVg/bSb85PhF+fZDU/fL9YxVd28pc8b2b9YVXkaUhsTmvFaxwbPNtRkfH
# iBahHb7t34txwYK8qDAs0TfqDZ2Qx7eU7KMLeb6GZs1a4kxvZw9wxs7WxHouggjJ
# H4Uhgm8jgK2JffMJa3kq6RMEJzel
# SIG # End signature block
