. ${PSScriptRoot}\def_upgrade.ps1

${active_mq_loc_final} = "${ACTIVE_MQ_DEFAULT}"
${java_loc_final} = "${JAVA_DEFAULT}"
${postgresql_loc_final} = "${POSTGRESQL_PATH_DEFAULT}"
${tomcat_loc_final} = "${TOMCAT_DEFAULT}"

${postgresql_admin} = "${POSTGRESQL_ADMIN_DEFAULT}"
${postgresql_port} = "${POSTGRESQL_PORT_DEFAULT}"
${postgresql_server} = "${POSTGRESQL_SERVER_DEFAULT}"

${active_mq_summary_code} = ${SUMMARY_CODE_FAILED}
${java_summary_code} = ${SUMMARY_CODE_FAILED}
${postgresql_summary_code} = ${SUMMARY_CODE_FAILED}
${tomcat_summary_code} = ${SUMMARY_CODE_FAILED}


If (Test-Path ${FILE_TOMCAT}) {
  ${tomcat_summary_code} = ${SUMMARY_CODE_SKIPPED}
  ${do_please} = Read-Host "It appears you are ready to upgrade Tomcat. Shall we proceed? [y/N]"
  If ("${do_please}" -eq "y" -Or "${do_please}" -eq "Y") {
    ${tomcat_summary_code} = ${SUMMARY_CODE_FAILED}
    ${input_invalid} = ${TRUE}
    while (${input_invalid}) {
      ${tomcat_loc} = Read-Host "  Where is your Tomcat located? [${tomcat_loc_final}]"
      If ("${tomcat_loc}" -ne "") { ${tomcat_loc_final} = ${tomcat_loc} }
      ${result} = (Get-PathValidity ${tomcat_loc_final} ${TOMCAT_VALIDITY})
      If ($result -eq ${TRUE} ) {
        ${input_invalid} = ${FALSE}
        Wait-StopProcesses "Tomcat" "tomcat"

        # Rename older tomcat parent folder
        ${tomcat_loc_rename} = "${tomcat_loc_final}-${TIMESTAMP}"
        Write-Host "    Renaming existing Tomcat location as `"${tomcat_loc_rename}`""
        Move-Item -Path ${tomcat_loc_final} -Destination ${tomcat_loc_rename} -Force

        # Unpack the new Tomcat tar where the old Tomcat was
        Unzip ${FILE_TOMCAT} ${tomcat_loc_final}

        Discard-SpecificFiles "Tomcat" ${tomcat_loc_final} ${TOMCAT_DISCARD}
        Copy-OldFiles "Tomcat" ${tomcat_loc_rename} ${tomcat_loc_final} ${TOMCAT_RETAIN}
        Disable-AJPConnector ${tomcat_loc_final}

        ${tomcat_summary_code} = ${SUMMARY_CODE_COMPLETED}
        Show-Differences ${tomcat_loc_final} ${tomcat_loc_rename}
      }
    }
  }
} Else {
  Write-Host "  You need to supply '${FILE_TOMCAT}' before Tomcat can be upgraded"
  ${tomcat_summary_code} = ${SUMMARY_CODE_MISSED}
}


If (Test-Path ${FILE_ACTIVE_MQ}) {
  ${active_mq_summary_code} = ${SUMMARY_CODE_SKIPPED}
  ${do_please} = Read-Host "It appears you are ready to upgrade ActiveMQ. Shall we proceed? [y/N]"
  If ("${do_please}" -eq "y" -Or "${do_please}" -eq "Y") {
    ${active_mq_summary_code} = ${SUMMARY_CODE_FAILED}
    ${input_invalid} = ${TRUE}
    while (${input_invalid}) {
      ${active_mq_loc} = Read-Host "  Where is your ActiveMQ located? [${active_mq_loc_final}]"
      If ("${active_mq_loc}" -ne "") { ${active_mq_loc_final} = ${active_mq_loc} }
      ${result} = (Get-PathValidity ${active_mq_loc_final} ${ACTIVE_MQ_VALIDITY})
      If ($result -eq ${TRUE} ) {
        ${input_invalid} = ${FALSE}
        Get-ValidJavaPath
        Wait-StopProcesses "ActiveMQ" "activemq"

        # Rename older ActiveMQ parent folder
        ${active_mq_loc_rename} = "${active_mq_loc_final}-${TIMESTAMP}"
        Write-Host "    Renaming existing ActiveMQ location as `"${active_mq_loc_rename}`""
        Move-Item -Path ${active_mq_loc_final} -Destination ${active_mq_loc_rename} -Force

        # Unpack the new ActiveMQ tar where the old ActiveMQ was
        Unzip ${FILE_ACTIVE_MQ} ${active_mq_loc_final}

        # Store Java location within activemq.bat
        ((Get-Content -path "${active_mq_loc_final}\bin\activemq.bat" -Raw) -replace
          "if `"%JAVA_HOME%`" == `"`" goto noJavaHome",
          "if `"%JAVA_HOME%`" == `"`" set JAVA_HOME=${java_loc_final}") |
          Set-Content -Path "${active_mq_loc_final}\bin\activemq.bat"

        Copy-OldFiles "ActiveMQ" ${active_mq_loc_rename} ${active_mq_loc_final} ${ACTIVE_MQ_RETAIN}

        ${active_mq_summary_code} = ${SUMMARY_CODE_COMPLETED}
        Show-Differences ${active_mq_loc_final} ${active_mq_loc_rename}
      }
    }
  }
} Else {
  Write-Host "  You need to supply '${FILE_ACTIVE_MQ}' before ActiveMQ can be upgraded"
  ${active_mq_summary_code} = ${SUMMARY_CODE_MISSED}
}


If (Test-Path ${FILE_JAVA}) {
  ${java_summary_code} = ${SUMMARY_CODE_SKIPPED}
  ${do_please} = Read-Host "It appears you are ready to upgrade Java. Shall we proceed? [y/N]"
  If ("${do_please}" -eq "y" -Or "${do_please}" -eq "Y") {
    ${java_summary_code} = ${SUMMARY_CODE_FAILED}
    Get-ValidJavaPath

    Wait-StopProcesses "Tomcat" "tomcat"
    Wait-StopProcesses "ActiveMQ" "activemq"

    # Make a link for copying files
    ${parent} = "$(Split-Path -Path ${java_loc_final} -Parent)"
    ${copy_link} = "${parent}\copier-${TIMESTAMP}"

    # Create a symbolic link for copying files
    ${item} = (Get-Item -Path ${java_loc_final})
    If (${item}.Attributes -match "ReparsePoint") {
      # A link will ultimately point to the new location
      ${copy_item} = (New-Item -ItemType SymbolicLink -Path "${copy_link}" -Target "$(Get-ResolvedPath ${item}.FullName)")
      ${item}.Delete()
    } Else {
      # A directory will need to be renamed since a link will replace it
      ${java_loc_rename} = "${java_loc_final}-${TIMESTAMP}"
      Rename-Item -Path ${java_loc_final} -NewName ${java_loc_rename} -Force
      ${copy_item} = (New-Item -ItemType SymbolicLink -Path "${copy_link}" -Target "${java_loc_rename}")
    }

    # Unpack Java into unique destination name (same path; jdk-timestamp)
    ${java_loc_jdk} = "${parent}\jdk"
    If (Test-Path ${java_loc_jdk}) {
      ${java_loc_jdk} = "${java_loc_jdk}-${TIMESTAMP}"
    }

    # Unpack the new Java tar where the old Java was
    Unzip ${FILE_JAVA} ${java_loc_jdk}

    # Create a symbolic link with the old name but new destination
    $null = (New-Item -ItemType SymbolicLink -Path "${java_loc_final}" -Target "${java_loc_jdk}\jre")
    Copy-OldFiles "Java" ${copy_link} ${java_loc_final} ${JAVA_RETAIN}

    ${java_summary_code} = ${SUMMARY_CODE_COMPLETED}
    Show-Differences "${java_loc_final}" "$(${copy_item}.Target)"

    # Remove copy-link
    ${copy_item}.Delete()
  }
} Else {
  Write-Host "  You need to supply '${FILE_JAVA}' before Java can be upgraded"
  ${java_summary_code} = ${SUMMARY_CODE_MISSED}
}


If (Test-Path ${FILE_POSTGRESQL}) {
  ${postgresql_summary_code} = ${SUMMARY_CODE_SKIPPED}
  ${do_please} = Read-Host "It appears you are ready to upgrade PostgreSQL. Shall we proceed? [y/N]"
  If ("${do_please}" -eq "y" -Or "${do_please}" -eq "Y") {
    ${postgresql_summary_code} = ${SUMMARY_CODE_FAILED}
    ${input_invalid} = ${TRUE}
    while (${input_invalid}) {
      ${postgresql_loc} = Read-Host "  Where is your PostgreSQL located? [${postgresql_loc_final}]"
      If ("${postgresql_loc}" -ne "") { ${postgresql_loc_final} = ${postgresql_loc} }
      ${result} = (Get-PathValidity ${postgresql_loc_final} ${POSTGRESQL_VALIDITY})
      If ($result -eq ${TRUE} ) {
        ${input_invalid} = ${FALSE}

        # Back up existing PostgreSQL data
        ${admin_temp} = Read-Host -Prompt "  Specify PostgreSQL admin username [${postgresql_admin}]"
        If ((${admin_temp} -ne ${null}) -and (${admin_temp}) -ne "") { ${postgresql_admin} = ${admin_temp} }

        ${private:PG_PWD_NEEDS_REMOVAL} = ${false}
        If (${env:PGPASSWORD} -eq ${null}) {
          ${private:PG_PWD_NEEDS_REMOVAL} = ${true}
          ${env:PGPASSWORD} = [System.Runtime.InteropServices.Marshal]::PtrToStringAuto(
            [System.Runtime.InteropServices.Marshal]::SecureStringToBSTR(
              $(Read-Host -Prompt "  Specify PostgreSQL admin password" -AsSecureString)
            )
          )
        } Else {
          Write-Host "  Reading PostgreSQL admin password from `"PGPASSWORD`" environment variable"
        }

        ${server_temp} = Read-Host -Prompt "  Specify PostgreSQL server [${postgresql_server}]"
        If ((${server_temp} -ne ${null}) -and (${server_temp}) -ne "") { ${postgresql_server} = ${server_temp} }

        ${port_temp} = Read-Host -Prompt "  Specify PostgreSQL port [${postgresql_port}]"
        If ((${port_temp} -ne ${null}) -and (${port_temp}) -ne "") { ${postgresql_port} = ${port_temp} }

        ${destination_files} = "${PSScriptRoot}\sql-${TIMESTAMP}"

        Backup-Databases `
          ${postgresql_admin} `
          ${postgresql_server} `
          ${postgresql_port} `
          ${postgresql_loc_final} `
          ${destination_files} `
          ${POSTGRESQL_DATABASES}

        # Stop and disable old PostgreSQL service
        Wait-StopServices "PostgreSQL" "postgres"
        Wait-DisableServices "PostgreSQL" "postgres"

        # Rename old PostgreSQL folder
        ${postgresql_loc_rename} = "${postgresql_loc_final}-${TIMESTAMP}"
        Write-Host "    Renaming existing PostgreSQL location as `"${postgresql_loc_rename}`""
        Move-Item -Path ${postgresql_loc_final} -Destination ${postgresql_loc_rename} -Force

        # Install PostgreSQL
        Write-Host "    Installing PostgreSQL using `"${FILE_POSTGRESQL}`""
        Write-Host (
          "    ",
          "Start-Process",
            "-FilePath ${FILE_POSTGRESQL}",
            "-Wait",
            "-NoNewWindow",
            "-ArgumentList",
              "`"--unattendedmodeui none`",",
              "`"--mode unattended`",",
              "`"--superaccount ${postgresql_admin}`",",
              "`"--superpassword *******`",",
              "`"--serverport ${postgresql_port}`",",
              "`"--prefix ${postgresql_loc_final}`",",
              "`"--datadir ${postgresql_loc_final}\data`""
        )
        Start-Process `
          -FilePath ${FILE_POSTGRESQL} `
          -Wait `
          -NoNewWindow `
          -ArgumentList `
            "--unattendedmodeui none", `
            "--mode unattended", `
            "--superaccount ${postgresql_admin}", `
            "--superpassword ${env:PGPASSWORD}", `
            "--serverport ${postgresql_port}", `
            "--prefix ${postgresql_loc_final}", `
            "--datadir ${postgresql_loc_final}\data"

        # Restore PostgreSQL data and files
        Copy-OldFiles "PostgreSQL" ${postgresql_loc_rename} ${postgresql_loc_final} ${POSTGRESQL_RETAIN_BIN}

        # Restart PostgreSQL service
        Get-Service -DisplayName "*postgres*" | Where-Object {($_.StartType -ne "Disable")} | Restart-Service -Force

        # Restore databases
        Restore-Databases `
          ${postgresql_admin} `
          ${postgresql_server} `
          ${postgresql_port} `
          ${postgresql_loc_final} `
          ${destination_files} `
          ${POSTGRESQL_DATABASES}

        # Remove PostgreSQL admin password if we set it
        If (${PG_PWD_NEEDS_REMOVAL}) {
          [System.Environment]::SetEnvironmentVariable("PGPASSWORD", $null)
        }

        ${postgresql_summary_code} = ${SUMMARY_CODE_COMPLETED}
        Show-Differences ${postgresql_loc_final} ${postgresql_loc_rename}
      }
    }
  }
} Else {
  Write-Host "  You need to supply '${FILE_POSTGRESQL}' before PostgreSQL can be upgraded"
  ${postgresql_summary_code} = ${SUMMARY_CODE_MISSED}
}


Summary-Register @("activemq", "java", "tomcat", "postgresql")
Summary-Print


# SIG # Begin signature block
# MIIggQYJKoZIhvcNAQcCoIIgcjCCIG4CAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCAqOg7w9Q75KqzL
# InQvaHxvI2DrU7e3nuq1ZuG59r+eD6CCG4cwggPFMIICraADAgECAhACrFwmagtA
# m48LefKuRiV3MA0GCSqGSIb3DQEBBQUAMGwxCzAJBgNVBAYTAlVTMRUwEwYDVQQK
# EwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xKzApBgNV
# BAMTIkRpZ2lDZXJ0IEhpZ2ggQXNzdXJhbmNlIEVWIFJvb3QgQ0EwHhcNMDYxMTEw
# MDAwMDAwWhcNMzExMTEwMDAwMDAwWjBsMQswCQYDVQQGEwJVUzEVMBMGA1UEChMM
# RGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29tMSswKQYDVQQD
# EyJEaWdpQ2VydCBIaWdoIEFzc3VyYW5jZSBFViBSb290IENBMIIBIjANBgkqhkiG
# 9w0BAQEFAAOCAQ8AMIIBCgKCAQEAxszlc+b71LvlLS0ypt/lgT/JzSVJtnEqw9WU
# NGeiChywX2mmQLHEt7KP0JikqUFZOtPclNY823Q4pErMTSWC90qlUxI47vNJbXGR
# fmO2q6Zfw6SE+E9iUb74xezbOJLjBuUIkQzEKEFV+8taiRV+ceg1v01yCT2+OjhQ
# W3cxG42zxyRFmqesbQAUWgS3uhPrUQqYQUEiTmVhh4FBUKZ5XIneGUpX1S7mXRxT
# LH6YzRoGFqRoc9A0BBNcoXHTWnxV215k4TeHMFYE5RG0KYAS8Xk5iKICEXwnZreI
# t3jyygqoOKsKZMK/Zl2VhMGhJR6HXRpQCyASzEG7bgtROLhLywIDAQABo2MwYTAO
# BgNVHQ8BAf8EBAMCAYYwDwYDVR0TAQH/BAUwAwEB/zAdBgNVHQ4EFgQUsT7DaQP4
# v0cB1JgmGggC72NkK8MwHwYDVR0jBBgwFoAUsT7DaQP4v0cB1JgmGggC72NkK8Mw
# DQYJKoZIhvcNAQEFBQADggEBABwaBpfc15yfPIhmBghXIdshR/gqZ6q/GDJ2QBBX
# wYrzetkRZY41+p78RbWe2UwxS7iR6EMsjrN4ztvjU3lx1uUhlAHaVYeaJGT2imbM
# 3pw3zag0sWmbI8ieeCIrcEPjVUcxYRnvWMWFL04w9qAxFiPI5+JlFjPLvxoboD34
# yl6LMYtgCIktDAZcUrfE+QqY0RVfnxK+fDZjOL1EpH/kJisKxJdpDemM4sAQV7jI
# dhKRVfJIadi8KgJbD0TUIDHb9LpwJl2QYJ68SxcJL7TLHkNoyQcnwdJc9+ohuWgS
# nDycv578gFybY83sR6olJ2egN/MAgn1U16n46S4To3foH0owggUoMIIEEKADAgEC
# AhANNvCcqoPZqIxHb9ZBXrLvMA0GCSqGSIb3DQEBCwUAMHYxCzAJBgNVBAYTAlVT
# MRUwEwYDVQQKEwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5j
# b20xNTAzBgNVBAMTLERpZ2lDZXJ0IFNIQTIgSGlnaCBBc3N1cmFuY2UgQ29kZSBT
# aWduaW5nIENBMB4XDTE4MTEzMDAwMDAwMFoXDTIxMTIwODEyMDAwMFowZzELMAkG
# A1UEBhMCVVMxDjAMBgNVBAgTBVRleGFzMRAwDgYDVQQHEwdIb3VzdG9uMRowGAYD
# VQQKExFOZXRJUSBDb3Jwb3JhdGlvbjEaMBgGA1UEAxMRTmV0SVEgQ29ycG9yYXRp
# b24wggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQCeZBYzhJbJO6boi5Tm
# q+1hqQvSRI1lsJZx5CqI3uxWjgFpDyaGGaNeXCAYAcw/YXPNt+wyDMiRqWQ2bz33
# b/EkLDlNZaJUeJVSzG/JBlm8Wtgu+veqi9bph+pBR9cKOYz6cQG3GxD4SPU/zIhD
# wwt/M963pOaVkt4kl56jWWPjRQTnb71KWsyxP/c/Z5WsbSH+QOqur/JayO4LkH/i
# yHT44R+Xb5OK9/XAh3eFmYbFxgwudwx4kkpiS6ckneCzRE+SN7JyvdTS+lThopN8
# 5K41BgY2Edc+k0628AiSo/xuPpzimOn+Hxouax9MfXShWT6Hh+QNNFI/2h7QTS6Y
# t3uBAgMBAAGjggG/MIIBuzAfBgNVHSMEGDAWgBRnnQ8gCQzMijrlgkZyYvzxzJDl
# QDAdBgNVHQ4EFgQU4oPg/2aI0v0mcRoEwVgbZEHhuRYwDgYDVR0PAQH/BAQDAgeA
# MBMGA1UdJQQMMAoGCCsGAQUFBwMDMG0GA1UdHwRmMGQwMKAuoCyGKmh0dHA6Ly9j
# cmwzLmRpZ2ljZXJ0LmNvbS9zaGEyLWhhLWNzLWcxLmNybDAwoC6gLIYqaHR0cDov
# L2NybDQuZGlnaWNlcnQuY29tL3NoYTItaGEtY3MtZzEuY3JsMEwGA1UdIARFMEMw
# NwYJYIZIAYb9bAMLMCowKAYIKwYBBQUHAgEWHGh0dHBzOi8vd3d3LmRpZ2ljZXJ0
# LmNvbS9DUFMwCAYGZ4EMAQQBMIGIBggrBgEFBQcBAQR8MHowJAYIKwYBBQUHMAGG
# GGh0dHA6Ly9vY3NwLmRpZ2ljZXJ0LmNvbTBSBggrBgEFBQcwAoZGaHR0cDovL2Nh
# Y2VydHMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0U0hBMkhpZ2hBc3N1cmFuY2VDb2Rl
# U2lnbmluZ0NBLmNydDAMBgNVHRMBAf8EAjAAMA0GCSqGSIb3DQEBCwUAA4IBAQBz
# dAg+QLbVAoX48OX0aZWGxUxBt/VCvwfWvyo0Z81E32M97dPS+z8rDPOQcD5cFmI/
# +GO3VAENqbAYuMN+OMq8qTqXU3EnmJekiF4rqIBSRBDUyYTrDvctyp/uM0DlmXcy
# sXEMEIa4t3UzEu0Ul/GAovRZVJ0ljQEAPsUP0JihcO7vakhfmP7B6AvKhYIUIJIR
# av7ML1aizJcKeV8lj1gDuSrG3KXqxGKnGSJnT1930YVJi8dmMvbx1AUmrEpiIQ6H
# LkKf1xAexMzAm/tpFp7W6vNfhST/UqkaJqOoR8FuhMmrCcCqjgKvlMjmMJaYkQ0X
# X/4fpBGqjWkNrFxnIuCAMIIFTzCCBDegAwIBAgIQC34QkDw4SQ/6L2eah6GnuTAN
# BgkqhkiG9w0BAQsFADBsMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQg
# SW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29tMSswKQYDVQQDEyJEaWdpQ2Vy
# dCBIaWdoIEFzc3VyYW5jZSBFViBSb290IENBMB4XDTEzMTAyMjEyMDAwMFoXDTI4
# MTAyMjEyMDAwMFowdjELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IElu
# YzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTE1MDMGA1UEAxMsRGlnaUNlcnQg
# U0hBMiBIaWdoIEFzc3VyYW5jZSBDb2RlIFNpZ25pbmcgQ0EwggEiMA0GCSqGSIb3
# DQEBAQUAA4IBDwAwggEKAoIBAQC0Sl59Bw9B3sT1dhY2vXH/zz9Pc0uc0Q3+SstX
# WF6FFt0CFVSZ8I88L00CeBBoyNg1Sz/B92fOmByuM7ktHaQKVJPEhaLfNbH18Tyn
# szT7XUjJRsliRLxImesoSVPDPY/ADt41mOliUd89a0Bh7gRB2s+nXFaW0flMt0SE
# h5hp5YK5E+ZVv8iScJIKMW9/izKrz2tan2LEPu6+7VmkU38L8VKIinsKZyTLkM3s
# 0k00TLDhtZ+cxvZvLM3mylN0AZ9nNd44SS3O7TlEghl5Thqytfu7ePBJZqfP+lyW
# dZKLGnLZ/1CSU8w+wkMyCRqGE2k8+4EyMzJkdXMoJh0IMDsHAgMBAAGjggHhMIIB
# 3TASBgNVHRMBAf8ECDAGAQH/AgEAMA4GA1UdDwEB/wQEAwIBhjATBgNVHSUEDDAK
# BggrBgEFBQcDAzB/BggrBgEFBQcBAQRzMHEwJAYIKwYBBQUHMAGGGGh0dHA6Ly9v
# Y3NwLmRpZ2ljZXJ0LmNvbTBJBggrBgEFBQcwAoY9aHR0cDovL2NhY2VydHMuZGln
# aWNlcnQuY29tL0RpZ2lDZXJ0SGlnaEFzc3VyYW5jZUVWUm9vdENBLmNydDCBjwYD
# VR0fBIGHMIGEMECgPqA8hjpodHRwOi8vY3JsNC5kaWdpY2VydC5jb20vRGlnaUNl
# cnRIaWdoQXNzdXJhbmNlRVZSb290Q0EuY3JsMECgPqA8hjpodHRwOi8vY3JsMy5k
# aWdpY2VydC5jb20vRGlnaUNlcnRIaWdoQXNzdXJhbmNlRVZSb290Q0EuY3JsME8G
# A1UdIARIMEYwOAYKYIZIAYb9bAACBDAqMCgGCCsGAQUFBwIBFhxodHRwczovL3d3
# dy5kaWdpY2VydC5jb20vQ1BTMAoGCGCGSAGG/WwDMB0GA1UdDgQWBBRnnQ8gCQzM
# ijrlgkZyYvzxzJDlQDAfBgNVHSMEGDAWgBSxPsNpA/i/RwHUmCYaCALvY2QrwzAN
# BgkqhkiG9w0BAQsFAAOCAQEAag7/fhN8BqVLwC6M+VNkCeK6WJEwUOzMn+HTqC9I
# RjYYKdB4KF+YVkAPHrq9sTuHXNxb2CAN7RoWTdURJCFL8SdpkBPrEaEB2v21TnlZ
# db04KmrD9o5BK4qii9csUVHZnKDI4066bKhH0k7RaB+MAlc7sylqjmogKrnyAGJk
# usjpAPnMpNS6mjXYryxlbBZ8WCHeSjDQ+uskXQbJnRa3rUpF0yXiDPBAqlxNrH7N
# BoK5dkZpCNgytoL+46lYNEMbjmdnlz9oMRY2OJU+h/fHw6+dencZ2d6Ttf1uK/yU
# +T23TBI1LDC+6I2eBXCaSBP0jNbnHqw456jzrQy3euxn7TCCBmowggVSoAMCAQIC
# EAMBmgI6/1ixa9bV6uYX8GYwDQYJKoZIhvcNAQEFBQAwYjELMAkGA1UEBhMCVVMx
# FTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNv
# bTEhMB8GA1UEAxMYRGlnaUNlcnQgQXNzdXJlZCBJRCBDQS0xMB4XDTE0MTAyMjAw
# MDAwMFoXDTI0MTAyMjAwMDAwMFowRzELMAkGA1UEBhMCVVMxETAPBgNVBAoTCERp
# Z2lDZXJ0MSUwIwYDVQQDExxEaWdpQ2VydCBUaW1lc3RhbXAgUmVzcG9uZGVyMIIB
# IjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAo2Rd/Hyz4II14OD2xirmSXU7
# zG7gU6mfH2RZ5nxrf2uMnVX4kuOe1VpjWwJJUNmDzm9m7t3LhelfpfnUh3SIRDsZ
# yeX1kZ/GFDmsJOqoSyyRicxeKPRktlC39RKzc5YKZ6O+YZ+u8/0SeHUOplsU/UUj
# joZEVX0YhgWMVYd5SEb3yg6Np95OX+Koti1ZAmGIYXIYaLm4fO7m5zQvMXeBMB+7
# NgGN7yfj95rwTDFkjePr+hmHqH7P7IwMNlt6wXq4eMfJBi5GEMiN6ARg27xzdPpO
# 2P6qQPGyznBGg+naQKFZOtkVCVeZVjCT88lhzNAIzGvsYkKRrALA76TwiRGPdwID
# AQABo4IDNTCCAzEwDgYDVR0PAQH/BAQDAgeAMAwGA1UdEwEB/wQCMAAwFgYDVR0l
# AQH/BAwwCgYIKwYBBQUHAwgwggG/BgNVHSAEggG2MIIBsjCCAaEGCWCGSAGG/WwH
# ATCCAZIwKAYIKwYBBQUHAgEWHGh0dHBzOi8vd3d3LmRpZ2ljZXJ0LmNvbS9DUFMw
# ggFkBggrBgEFBQcCAjCCAVYeggFSAEEAbgB5ACAAdQBzAGUAIABvAGYAIAB0AGgA
# aQBzACAAQwBlAHIAdABpAGYAaQBjAGEAdABlACAAYwBvAG4AcwB0AGkAdAB1AHQA
# ZQBzACAAYQBjAGMAZQBwAHQAYQBuAGMAZQAgAG8AZgAgAHQAaABlACAARABpAGcA
# aQBDAGUAcgB0ACAAQwBQAC8AQwBQAFMAIABhAG4AZAAgAHQAaABlACAAUgBlAGwA
# eQBpAG4AZwAgAFAAYQByAHQAeQAgAEEAZwByAGUAZQBtAGUAbgB0ACAAdwBoAGkA
# YwBoACAAbABpAG0AaQB0ACAAbABpAGEAYgBpAGwAaQB0AHkAIABhAG4AZAAgAGEA
# cgBlACAAaQBuAGMAbwByAHAAbwByAGEAdABlAGQAIABoAGUAcgBlAGkAbgAgAGIA
# eQAgAHIAZQBmAGUAcgBlAG4AYwBlAC4wCwYJYIZIAYb9bAMVMB8GA1UdIwQYMBaA
# FBUAEisTmLKZB+0e36K+Vw0rZwLNMB0GA1UdDgQWBBRhWk0ktkkynUoqeRqDS/Qe
# icHKfTB9BgNVHR8EdjB0MDigNqA0hjJodHRwOi8vY3JsMy5kaWdpY2VydC5jb20v
# RGlnaUNlcnRBc3N1cmVkSURDQS0xLmNybDA4oDagNIYyaHR0cDovL2NybDQuZGln
# aWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEQ0EtMS5jcmwwdwYIKwYBBQUHAQEE
# azBpMCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2VydC5jb20wQQYIKwYB
# BQUHMAKGNWh0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3Vy
# ZWRJRENBLTEuY3J0MA0GCSqGSIb3DQEBBQUAA4IBAQCdJX4bM02yJoFcm4bOIyAP
# gIfliP//sdRqLDHtOhcZcRfNqRu8WhY5AJ3jbITkWkD73gYBjDf6m7GdJH7+IKRX
# rVu3mrBgJuppVyFdNC8fcbCDlBkFazWQEKB7l8f2P+fiEUGmvWLZ8Cc9OB0obzpS
# CfDscGLTYkuw4HOmksDTjjHYL+NtFxMG7uQDthSr849Dp3GdId0UyhVdkkHa+Q+B
# 0Zl0DSbEDn8btfWg8cZ3BigV6diT5VUW8LsKqxzbXEgnZsijiwoc5ZXarsQuWaBh
# 3drzbaJh6YoLbewSGL33VVRAA5Ira8JRwgpIr7DUbuD0FAo6G+OPPcqvao173NhE
# MIIGzTCCBbWgAwIBAgIQBv35A5YDreoACus/J7u6GzANBgkqhkiG9w0BAQUFADBl
# MQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3
# d3cuZGlnaWNlcnQuY29tMSQwIgYDVQQDExtEaWdpQ2VydCBBc3N1cmVkIElEIFJv
# b3QgQ0EwHhcNMDYxMTEwMDAwMDAwWhcNMjExMTEwMDAwMDAwWjBiMQswCQYDVQQG
# EwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNl
# cnQuY29tMSEwHwYDVQQDExhEaWdpQ2VydCBBc3N1cmVkIElEIENBLTEwggEiMA0G
# CSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQDogi2Z+crCQpWlgHNAcNKeVlRcqcTS
# QQaPyTP8TUWRXIGf7Syc+BZZ3561JBXCmLm0d0ncicQK2q/LXmvtrbBxMevPOkAM
# Rk2T7It6NggDqww0/hhJgv7HxzFIgHweog+SDlDJxofrNj/YMMP/pvf7os1vcyP+
# rFYFkPAyIRaJxnCI+QWXfaPHQ90C6Ds97bFBo+0/vtuVSMTuHrPyvAwrmdDGXRJC
# geGDboJzPyZLFJCuWWYKxI2+0s4Grq2Eb0iEm09AufFM8q+Y+/bOQF1c9qjxL6/s
# iSLyaxhlscFzrdfx2M8eCnRcQrhofrfVdwonVnwPYqQ/MhRglf0HBKIJAgMBAAGj
# ggN6MIIDdjAOBgNVHQ8BAf8EBAMCAYYwOwYDVR0lBDQwMgYIKwYBBQUHAwEGCCsG
# AQUFBwMCBggrBgEFBQcDAwYIKwYBBQUHAwQGCCsGAQUFBwMIMIIB0gYDVR0gBIIB
# yTCCAcUwggG0BgpghkgBhv1sAAEEMIIBpDA6BggrBgEFBQcCARYuaHR0cDovL3d3
# dy5kaWdpY2VydC5jb20vc3NsLWNwcy1yZXBvc2l0b3J5Lmh0bTCCAWQGCCsGAQUF
# BwICMIIBVh6CAVIAQQBuAHkAIAB1AHMAZQAgAG8AZgAgAHQAaABpAHMAIABDAGUA
# cgB0AGkAZgBpAGMAYQB0AGUAIABjAG8AbgBzAHQAaQB0AHUAdABlAHMAIABhAGMA
# YwBlAHAAdABhAG4AYwBlACAAbwBmACAAdABoAGUAIABEAGkAZwBpAEMAZQByAHQA
# IABDAFAALwBDAFAAUwAgAGEAbgBkACAAdABoAGUAIABSAGUAbAB5AGkAbgBnACAA
# UABhAHIAdAB5ACAAQQBnAHIAZQBlAG0AZQBuAHQAIAB3AGgAaQBjAGgAIABsAGkA
# bQBpAHQAIABsAGkAYQBiAGkAbABpAHQAeQAgAGEAbgBkACAAYQByAGUAIABpAG4A
# YwBvAHIAcABvAHIAYQB0AGUAZAAgAGgAZQByAGUAaQBuACAAYgB5ACAAcgBlAGYA
# ZQByAGUAbgBjAGUALjALBglghkgBhv1sAxUwEgYDVR0TAQH/BAgwBgEB/wIBADB5
# BggrBgEFBQcBAQRtMGswJAYIKwYBBQUHMAGGGGh0dHA6Ly9vY3NwLmRpZ2ljZXJ0
# LmNvbTBDBggrBgEFBQcwAoY3aHR0cDovL2NhY2VydHMuZGlnaWNlcnQuY29tL0Rp
# Z2lDZXJ0QXNzdXJlZElEUm9vdENBLmNydDCBgQYDVR0fBHoweDA6oDigNoY0aHR0
# cDovL2NybDMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9vdENBLmNy
# bDA6oDigNoY0aHR0cDovL2NybDQuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJl
# ZElEUm9vdENBLmNybDAdBgNVHQ4EFgQUFQASKxOYspkH7R7for5XDStnAs0wHwYD
# VR0jBBgwFoAUReuir/SSy4IxLVGLp6chnfNtyA8wDQYJKoZIhvcNAQEFBQADggEB
# AEZQPsm3KCSnOB22WymvUs9S6TFHq1Zce9UNC0Gz7+x1H3Q48rJcYaKclcNQ5IK5
# I9G6OoZyrTh4rHVdFxc0ckeFlFbR67s2hHfMJKXzBBlVqefj56tizfuLLZDCwNK1
# lL1eT7EF0g49GqkUW6aGMWKoqDPkmzmnxPXOHXh2lCVz5Cqrz5x2S+1fwksW5Etw
# TACJHvzFebxMElf+X+EevAJdqP77BzhPDcZdkbkPZ0XN1oPt55INjbFpjE/7WeAj
# D9KqrgB87pxCDs+R1ye3Fu4Pw718CqDuLAhVhSK46xgaTfwqIa1JMYNHlXdx3LEb
# S0scEJx3FMGdTy9alQgpECYxggRQMIIETAIBATCBijB2MQswCQYDVQQGEwJVUzEV
# MBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29t
# MTUwMwYDVQQDEyxEaWdpQ2VydCBTSEEyIEhpZ2ggQXNzdXJhbmNlIENvZGUgU2ln
# bmluZyBDQQIQDTbwnKqD2aiMR2/WQV6y7zANBglghkgBZQMEAgEFAKCBhDAYBgor
# BgEEAYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJAzEMBgorBgEEAYI3AgEE
# MBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEVMC8GCSqGSIb3DQEJBDEiBCCa
# Qwn51NNc0BdI9NssZQb/x4YwxZCQoYtFqL5zq5d1RzANBgkqhkiG9w0BAQEFAASC
# AQBUmbeh79YPP/VQb6e+HiAEhLVRJSgYMKWgmapIaFYLoDlbRqqE9nEnP6IlUgEb
# 7xWDqCYei+zhQiEhuXJwxM6GA1Z+OtfIhlqF7/h5Sx86oscoFak77JKZb6/XAE2T
# OYRC8n0HpfoZijzBP+V3DzJ3Pp3JGbCRALW9/7IxDP5a6xcagZ5CKOShKPwHLFgM
# yG76b0PE/UP4qTHJ8zRMjKNmRJQWfQRGiVsZtI9uo0siZCRrwinuxnUUWSOqupMh
# gzAKIsRBAfDq599ENEQzLKQ9KeWzw0wtGK2lrE9bQKsjN2EygAYVxc8looZ2zXIL
# 0VDi1o/zlrNucJAJIf+9T5sJoYICDzCCAgsGCSqGSIb3DQEJBjGCAfwwggH4AgEB
# MHYwYjELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UE
# CxMQd3d3LmRpZ2ljZXJ0LmNvbTEhMB8GA1UEAxMYRGlnaUNlcnQgQXNzdXJlZCBJ
# RCBDQS0xAhADAZoCOv9YsWvW1ermF/BmMAkGBSsOAwIaBQCgXTAYBgkqhkiG9w0B
# CQMxCwYJKoZIhvcNAQcBMBwGCSqGSIb3DQEJBTEPFw0yMDA0MjEwMTA4MTVaMCMG
# CSqGSIb3DQEJBDEWBBQdjWKfKD6iC3oK/2QbTQeEWxO/bjANBgkqhkiG9w0BAQEF
# AASCAQCRYiWPIFLQ+8yQVnedF032V58etb9gUoAxDlHrZHqqbQtiL65YkEnGXr58
# ASanqZ/RIQhbJFxkHsVrYvulP88MJHLHskYi4117wY1hm7R+u0vC+lpoXUyibjYC
# aHwXYGgPYWPUo0cx/1F5oLFarNwTD0x+DtAgSr+GzDUYhT2t8RbfK0oY7ITk6oQN
# erkX84dklQa9N9XzKq25nFd6bTG0uKE9WnHMhpJSVuEkyDYrQhqeOI3SVv+We/ey
# 4YRs3FqvSe5d8gCdmrMqNF+nKuIab8mtLb34zsWIc7zyPaJWFDQFnhvYdVZeijtJ
# HOwCkGJZ0xraNDBxdxwaoSU10OJe
# SIG # End signature block
