. ${PSScriptRoot}\def_upgrade.ps1

${active_mq_loc_final} = "${ACTIVE_MQ_DEFAULT}"
${java_loc_final} = "${JAVA_DEFAULT}"
${postgresql_loc_final} = "${POSTGRESQL_PATH_DEFAULT}"
${tomcat_loc_final} = "${TOMCAT_DEFAULT}"

${do_please} = Read-Host "Do you want to compare Tomcat versions? [Y/n]"
If ("${do_please}" -eq "" -Or "${do_please}" -eq "y" -Or "${do_please}" -eq "Y") {
  ${input_invalid} = ${TRUE}
  while (${input_invalid}) {
    ${tomcat_loc} = Read-Host "  Where is your Tomcat located? [${tomcat_loc_final}]"
    If ("${tomcat_loc}" -ne "") { ${tomcat_loc_final} = ${tomcat_loc} }
    ${result} = (Get-PathValidity ${tomcat_loc_final} ${TOMCAT_VALIDITY})
    If ($result -eq ${TRUE}) {
      ${input_invalid} = ${FALSE}
      ${tomcat_version_output} = (cmd /c "${tomcat_loc_final}\bin\setenv.bat & ${tomcat_loc_final}\bin\version.bat")
      ${tomcat_version_exist} = (${tomcat_version_output} |
        Select-String -Pattern 'Server version:') -replace "Server version:\s+Apache Tomcat/", ""
      Compare-VersionsWithResult "Tomcat" ${tomcat_version_exist} ${TOMCAT_VERSION_TARGET}

      ${java_loc_final} = (${tomcat_version_output} |
        Select-String -Pattern 'Using JRE_HOME:') -replace 'Using JRE_HOME:\s+"', '' -replace '"$', ''
      If ("${java_loc_final}" -ne "") {
        ${global:ALREADY_OBTAINED_JAVA_LOC} = ${TRUE}
        Write-Host "  Tomcat is using Java located at `"${java_loc_final}`""
      }
    }
  }
}


${do_please} = Read-Host "Do you want to compare ActiveMQ versions? [y/N]"
If ("${do_please}" -eq "y" -Or "${do_please}" -eq "Y") {
  ${input_invalid} = ${TRUE}
  while (${input_invalid}) {
    ${active_mq_loc} = Read-Host "  Where is your ActiveMQ located? [${active_mq_loc_final}]"
    If ("${active_mq_loc}" -ne "") { ${active_mq_loc_final} = ${active_mq_loc} }
    ${result} = (Get-PathValidity ${active_mq_loc_final} ${ACTIVE_MQ_VALIDITY})
    If ($result -eq ${TRUE} ) {
      ${input_invalid} = ${FALSE}

      ${java_home_loc_activemq} = (Get-Content -Path "${active_mq_loc_final}\bin\activemq.bat" |
        Select-String -Pattern '.*set\sJAVA_HOME=' ) -replace ".*set\sJAVA_HOME=", "" -replace "`"", ""
      If ("${java_home_loc_activemq}" -ne "") {
        Write-Host "  ActiveMQ is using Java located at `"${java_home_loc_activemq}`""
        If (-not ${global:ALREADY_OBTAINED_JAVA_LOC}) {
          ${global:ALREADY_OBTAINED_JAVA_LOC} = ${TRUE}
          ${java_loc_final} = ${java_home_loc_activemq}
        }
      } Else {
        Get-ValidJavaPath
        ${global:JAVA_HOME} = ${java_loc_final}
      }
      ${active_mq_version_exist} = (cmd /c "${active_mq_loc_final}\bin\activemq --version" |
        Select-String -Pattern 'ActiveMQ\s') -replace "ActiveMQ\s+", ""
      Compare-VersionsWithResult "ActiveMQ" ${active_mq_version_exist} ${ACTIVE_MQ_VERSION_TARGET}
    }
  }
}


${do_please} = Read-Host "Do you want to compare Java versions? [Y/n]"
If ("${do_please}" -eq "" -Or "${do_please}" -eq "y" -Or "${do_please}" -eq "Y") {
  If (-not ${global:ALREADY_OBTAINED_JAVA_LOC}) {
    Get-ValidJavaPath
  }
  ${java_version_exist} = ((cmd /c "${java_loc_final}\bin\java.exe -version" 2>&1) |
    Select-String -Pattern "^.*\sversion") -replace "^.*\sversion\s+`"", "" -replace "`"", ""
  Compare-VersionsWithResult "Java" ${java_version_exist} ${JAVA_VERSION_TARGET}
}


${do_please} = Read-Host "Do you want to compare PostgreSQL versions? [y/N]"
If ("${do_please}" -eq "y" -Or "${do_please}" -eq "Y") {
  ${input_invalid} = ${TRUE}
  while (${input_invalid}) {
    ${postgresql_loc} = Read-Host "  Where is your PostgreSQL located? [${postgresql_loc_final}]"
    If ("${postgresql_loc}" -ne "") { ${postgresql_loc_final} = ${postgresql_loc} }
    ${result} = (Get-PathValidity ${postgresql_loc_final} ${POSTGRESQL_VALIDITY})
    If ($result -eq ${TRUE} ) {
      ${input_invalid} = ${FALSE}
      ${postgresql_version_exist} = (cmd /c "${postgresql_loc_final}\bin\psql --version" |
        Select-String -Pattern 'psql\s+\(PostgreSQL\)\s') -replace "psql\s+\(PostgreSQL\)\s+", ""
      Compare-VersionsWithResult "PostgreSQL" ${postgresql_version_exist} ${POSTGRESQL_VERSION_TARGET}
    }
  }
}


# SIG # Begin signature block
# MIIggQYJKoZIhvcNAQcCoIIgcjCCIG4CAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCCEyNBLy1XtjRhT
# bK6T39avKGH55hH8CahqiCh0S+7UA6CCG4cwggPFMIICraADAgECAhACrFwmagtA
# m48LefKuRiV3MA0GCSqGSIb3DQEBBQUAMGwxCzAJBgNVBAYTAlVTMRUwEwYDVQQK
# EwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xKzApBgNV
# BAMTIkRpZ2lDZXJ0IEhpZ2ggQXNzdXJhbmNlIEVWIFJvb3QgQ0EwHhcNMDYxMTEw
# MDAwMDAwWhcNMzExMTEwMDAwMDAwWjBsMQswCQYDVQQGEwJVUzEVMBMGA1UEChMM
# RGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29tMSswKQYDVQQD
# EyJEaWdpQ2VydCBIaWdoIEFzc3VyYW5jZSBFViBSb290IENBMIIBIjANBgkqhkiG
# 9w0BAQEFAAOCAQ8AMIIBCgKCAQEAxszlc+b71LvlLS0ypt/lgT/JzSVJtnEqw9WU
# NGeiChywX2mmQLHEt7KP0JikqUFZOtPclNY823Q4pErMTSWC90qlUxI47vNJbXGR
# fmO2q6Zfw6SE+E9iUb74xezbOJLjBuUIkQzEKEFV+8taiRV+ceg1v01yCT2+OjhQ
# W3cxG42zxyRFmqesbQAUWgS3uhPrUQqYQUEiTmVhh4FBUKZ5XIneGUpX1S7mXRxT
# LH6YzRoGFqRoc9A0BBNcoXHTWnxV215k4TeHMFYE5RG0KYAS8Xk5iKICEXwnZreI
# t3jyygqoOKsKZMK/Zl2VhMGhJR6HXRpQCyASzEG7bgtROLhLywIDAQABo2MwYTAO
# BgNVHQ8BAf8EBAMCAYYwDwYDVR0TAQH/BAUwAwEB/zAdBgNVHQ4EFgQUsT7DaQP4
# v0cB1JgmGggC72NkK8MwHwYDVR0jBBgwFoAUsT7DaQP4v0cB1JgmGggC72NkK8Mw
# DQYJKoZIhvcNAQEFBQADggEBABwaBpfc15yfPIhmBghXIdshR/gqZ6q/GDJ2QBBX
# wYrzetkRZY41+p78RbWe2UwxS7iR6EMsjrN4ztvjU3lx1uUhlAHaVYeaJGT2imbM
# 3pw3zag0sWmbI8ieeCIrcEPjVUcxYRnvWMWFL04w9qAxFiPI5+JlFjPLvxoboD34
# yl6LMYtgCIktDAZcUrfE+QqY0RVfnxK+fDZjOL1EpH/kJisKxJdpDemM4sAQV7jI
# dhKRVfJIadi8KgJbD0TUIDHb9LpwJl2QYJ68SxcJL7TLHkNoyQcnwdJc9+ohuWgS
# nDycv578gFybY83sR6olJ2egN/MAgn1U16n46S4To3foH0owggUoMIIEEKADAgEC
# AhANNvCcqoPZqIxHb9ZBXrLvMA0GCSqGSIb3DQEBCwUAMHYxCzAJBgNVBAYTAlVT
# MRUwEwYDVQQKEwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5j
# b20xNTAzBgNVBAMTLERpZ2lDZXJ0IFNIQTIgSGlnaCBBc3N1cmFuY2UgQ29kZSBT
# aWduaW5nIENBMB4XDTE4MTEzMDAwMDAwMFoXDTIxMTIwODEyMDAwMFowZzELMAkG
# A1UEBhMCVVMxDjAMBgNVBAgTBVRleGFzMRAwDgYDVQQHEwdIb3VzdG9uMRowGAYD
# VQQKExFOZXRJUSBDb3Jwb3JhdGlvbjEaMBgGA1UEAxMRTmV0SVEgQ29ycG9yYXRp
# b24wggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQCeZBYzhJbJO6boi5Tm
# q+1hqQvSRI1lsJZx5CqI3uxWjgFpDyaGGaNeXCAYAcw/YXPNt+wyDMiRqWQ2bz33
# b/EkLDlNZaJUeJVSzG/JBlm8Wtgu+veqi9bph+pBR9cKOYz6cQG3GxD4SPU/zIhD
# wwt/M963pOaVkt4kl56jWWPjRQTnb71KWsyxP/c/Z5WsbSH+QOqur/JayO4LkH/i
# yHT44R+Xb5OK9/XAh3eFmYbFxgwudwx4kkpiS6ckneCzRE+SN7JyvdTS+lThopN8
# 5K41BgY2Edc+k0628AiSo/xuPpzimOn+Hxouax9MfXShWT6Hh+QNNFI/2h7QTS6Y
# t3uBAgMBAAGjggG/MIIBuzAfBgNVHSMEGDAWgBRnnQ8gCQzMijrlgkZyYvzxzJDl
# QDAdBgNVHQ4EFgQU4oPg/2aI0v0mcRoEwVgbZEHhuRYwDgYDVR0PAQH/BAQDAgeA
# MBMGA1UdJQQMMAoGCCsGAQUFBwMDMG0GA1UdHwRmMGQwMKAuoCyGKmh0dHA6Ly9j
# cmwzLmRpZ2ljZXJ0LmNvbS9zaGEyLWhhLWNzLWcxLmNybDAwoC6gLIYqaHR0cDov
# L2NybDQuZGlnaWNlcnQuY29tL3NoYTItaGEtY3MtZzEuY3JsMEwGA1UdIARFMEMw
# NwYJYIZIAYb9bAMLMCowKAYIKwYBBQUHAgEWHGh0dHBzOi8vd3d3LmRpZ2ljZXJ0
# LmNvbS9DUFMwCAYGZ4EMAQQBMIGIBggrBgEFBQcBAQR8MHowJAYIKwYBBQUHMAGG
# GGh0dHA6Ly9vY3NwLmRpZ2ljZXJ0LmNvbTBSBggrBgEFBQcwAoZGaHR0cDovL2Nh
# Y2VydHMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0U0hBMkhpZ2hBc3N1cmFuY2VDb2Rl
# U2lnbmluZ0NBLmNydDAMBgNVHRMBAf8EAjAAMA0GCSqGSIb3DQEBCwUAA4IBAQBz
# dAg+QLbVAoX48OX0aZWGxUxBt/VCvwfWvyo0Z81E32M97dPS+z8rDPOQcD5cFmI/
# +GO3VAENqbAYuMN+OMq8qTqXU3EnmJekiF4rqIBSRBDUyYTrDvctyp/uM0DlmXcy
# sXEMEIa4t3UzEu0Ul/GAovRZVJ0ljQEAPsUP0JihcO7vakhfmP7B6AvKhYIUIJIR
# av7ML1aizJcKeV8lj1gDuSrG3KXqxGKnGSJnT1930YVJi8dmMvbx1AUmrEpiIQ6H
# LkKf1xAexMzAm/tpFp7W6vNfhST/UqkaJqOoR8FuhMmrCcCqjgKvlMjmMJaYkQ0X
# X/4fpBGqjWkNrFxnIuCAMIIFTzCCBDegAwIBAgIQC34QkDw4SQ/6L2eah6GnuTAN
# BgkqhkiG9w0BAQsFADBsMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQg
# SW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29tMSswKQYDVQQDEyJEaWdpQ2Vy
# dCBIaWdoIEFzc3VyYW5jZSBFViBSb290IENBMB4XDTEzMTAyMjEyMDAwMFoXDTI4
# MTAyMjEyMDAwMFowdjELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IElu
# YzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTE1MDMGA1UEAxMsRGlnaUNlcnQg
# U0hBMiBIaWdoIEFzc3VyYW5jZSBDb2RlIFNpZ25pbmcgQ0EwggEiMA0GCSqGSIb3
# DQEBAQUAA4IBDwAwggEKAoIBAQC0Sl59Bw9B3sT1dhY2vXH/zz9Pc0uc0Q3+SstX
# WF6FFt0CFVSZ8I88L00CeBBoyNg1Sz/B92fOmByuM7ktHaQKVJPEhaLfNbH18Tyn
# szT7XUjJRsliRLxImesoSVPDPY/ADt41mOliUd89a0Bh7gRB2s+nXFaW0flMt0SE
# h5hp5YK5E+ZVv8iScJIKMW9/izKrz2tan2LEPu6+7VmkU38L8VKIinsKZyTLkM3s
# 0k00TLDhtZ+cxvZvLM3mylN0AZ9nNd44SS3O7TlEghl5Thqytfu7ePBJZqfP+lyW
# dZKLGnLZ/1CSU8w+wkMyCRqGE2k8+4EyMzJkdXMoJh0IMDsHAgMBAAGjggHhMIIB
# 3TASBgNVHRMBAf8ECDAGAQH/AgEAMA4GA1UdDwEB/wQEAwIBhjATBgNVHSUEDDAK
# BggrBgEFBQcDAzB/BggrBgEFBQcBAQRzMHEwJAYIKwYBBQUHMAGGGGh0dHA6Ly9v
# Y3NwLmRpZ2ljZXJ0LmNvbTBJBggrBgEFBQcwAoY9aHR0cDovL2NhY2VydHMuZGln
# aWNlcnQuY29tL0RpZ2lDZXJ0SGlnaEFzc3VyYW5jZUVWUm9vdENBLmNydDCBjwYD
# VR0fBIGHMIGEMECgPqA8hjpodHRwOi8vY3JsNC5kaWdpY2VydC5jb20vRGlnaUNl
# cnRIaWdoQXNzdXJhbmNlRVZSb290Q0EuY3JsMECgPqA8hjpodHRwOi8vY3JsMy5k
# aWdpY2VydC5jb20vRGlnaUNlcnRIaWdoQXNzdXJhbmNlRVZSb290Q0EuY3JsME8G
# A1UdIARIMEYwOAYKYIZIAYb9bAACBDAqMCgGCCsGAQUFBwIBFhxodHRwczovL3d3
# dy5kaWdpY2VydC5jb20vQ1BTMAoGCGCGSAGG/WwDMB0GA1UdDgQWBBRnnQ8gCQzM
# ijrlgkZyYvzxzJDlQDAfBgNVHSMEGDAWgBSxPsNpA/i/RwHUmCYaCALvY2QrwzAN
# BgkqhkiG9w0BAQsFAAOCAQEAag7/fhN8BqVLwC6M+VNkCeK6WJEwUOzMn+HTqC9I
# RjYYKdB4KF+YVkAPHrq9sTuHXNxb2CAN7RoWTdURJCFL8SdpkBPrEaEB2v21TnlZ
# db04KmrD9o5BK4qii9csUVHZnKDI4066bKhH0k7RaB+MAlc7sylqjmogKrnyAGJk
# usjpAPnMpNS6mjXYryxlbBZ8WCHeSjDQ+uskXQbJnRa3rUpF0yXiDPBAqlxNrH7N
# BoK5dkZpCNgytoL+46lYNEMbjmdnlz9oMRY2OJU+h/fHw6+dencZ2d6Ttf1uK/yU
# +T23TBI1LDC+6I2eBXCaSBP0jNbnHqw456jzrQy3euxn7TCCBmowggVSoAMCAQIC
# EAMBmgI6/1ixa9bV6uYX8GYwDQYJKoZIhvcNAQEFBQAwYjELMAkGA1UEBhMCVVMx
# FTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNv
# bTEhMB8GA1UEAxMYRGlnaUNlcnQgQXNzdXJlZCBJRCBDQS0xMB4XDTE0MTAyMjAw
# MDAwMFoXDTI0MTAyMjAwMDAwMFowRzELMAkGA1UEBhMCVVMxETAPBgNVBAoTCERp
# Z2lDZXJ0MSUwIwYDVQQDExxEaWdpQ2VydCBUaW1lc3RhbXAgUmVzcG9uZGVyMIIB
# IjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAo2Rd/Hyz4II14OD2xirmSXU7
# zG7gU6mfH2RZ5nxrf2uMnVX4kuOe1VpjWwJJUNmDzm9m7t3LhelfpfnUh3SIRDsZ
# yeX1kZ/GFDmsJOqoSyyRicxeKPRktlC39RKzc5YKZ6O+YZ+u8/0SeHUOplsU/UUj
# joZEVX0YhgWMVYd5SEb3yg6Np95OX+Koti1ZAmGIYXIYaLm4fO7m5zQvMXeBMB+7
# NgGN7yfj95rwTDFkjePr+hmHqH7P7IwMNlt6wXq4eMfJBi5GEMiN6ARg27xzdPpO
# 2P6qQPGyznBGg+naQKFZOtkVCVeZVjCT88lhzNAIzGvsYkKRrALA76TwiRGPdwID
# AQABo4IDNTCCAzEwDgYDVR0PAQH/BAQDAgeAMAwGA1UdEwEB/wQCMAAwFgYDVR0l
# AQH/BAwwCgYIKwYBBQUHAwgwggG/BgNVHSAEggG2MIIBsjCCAaEGCWCGSAGG/WwH
# ATCCAZIwKAYIKwYBBQUHAgEWHGh0dHBzOi8vd3d3LmRpZ2ljZXJ0LmNvbS9DUFMw
# ggFkBggrBgEFBQcCAjCCAVYeggFSAEEAbgB5ACAAdQBzAGUAIABvAGYAIAB0AGgA
# aQBzACAAQwBlAHIAdABpAGYAaQBjAGEAdABlACAAYwBvAG4AcwB0AGkAdAB1AHQA
# ZQBzACAAYQBjAGMAZQBwAHQAYQBuAGMAZQAgAG8AZgAgAHQAaABlACAARABpAGcA
# aQBDAGUAcgB0ACAAQwBQAC8AQwBQAFMAIABhAG4AZAAgAHQAaABlACAAUgBlAGwA
# eQBpAG4AZwAgAFAAYQByAHQAeQAgAEEAZwByAGUAZQBtAGUAbgB0ACAAdwBoAGkA
# YwBoACAAbABpAG0AaQB0ACAAbABpAGEAYgBpAGwAaQB0AHkAIABhAG4AZAAgAGEA
# cgBlACAAaQBuAGMAbwByAHAAbwByAGEAdABlAGQAIABoAGUAcgBlAGkAbgAgAGIA
# eQAgAHIAZQBmAGUAcgBlAG4AYwBlAC4wCwYJYIZIAYb9bAMVMB8GA1UdIwQYMBaA
# FBUAEisTmLKZB+0e36K+Vw0rZwLNMB0GA1UdDgQWBBRhWk0ktkkynUoqeRqDS/Qe
# icHKfTB9BgNVHR8EdjB0MDigNqA0hjJodHRwOi8vY3JsMy5kaWdpY2VydC5jb20v
# RGlnaUNlcnRBc3N1cmVkSURDQS0xLmNybDA4oDagNIYyaHR0cDovL2NybDQuZGln
# aWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEQ0EtMS5jcmwwdwYIKwYBBQUHAQEE
# azBpMCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2VydC5jb20wQQYIKwYB
# BQUHMAKGNWh0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3Vy
# ZWRJRENBLTEuY3J0MA0GCSqGSIb3DQEBBQUAA4IBAQCdJX4bM02yJoFcm4bOIyAP
# gIfliP//sdRqLDHtOhcZcRfNqRu8WhY5AJ3jbITkWkD73gYBjDf6m7GdJH7+IKRX
# rVu3mrBgJuppVyFdNC8fcbCDlBkFazWQEKB7l8f2P+fiEUGmvWLZ8Cc9OB0obzpS
# CfDscGLTYkuw4HOmksDTjjHYL+NtFxMG7uQDthSr849Dp3GdId0UyhVdkkHa+Q+B
# 0Zl0DSbEDn8btfWg8cZ3BigV6diT5VUW8LsKqxzbXEgnZsijiwoc5ZXarsQuWaBh
# 3drzbaJh6YoLbewSGL33VVRAA5Ira8JRwgpIr7DUbuD0FAo6G+OPPcqvao173NhE
# MIIGzTCCBbWgAwIBAgIQBv35A5YDreoACus/J7u6GzANBgkqhkiG9w0BAQUFADBl
# MQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3
# d3cuZGlnaWNlcnQuY29tMSQwIgYDVQQDExtEaWdpQ2VydCBBc3N1cmVkIElEIFJv
# b3QgQ0EwHhcNMDYxMTEwMDAwMDAwWhcNMjExMTEwMDAwMDAwWjBiMQswCQYDVQQG
# EwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNl
# cnQuY29tMSEwHwYDVQQDExhEaWdpQ2VydCBBc3N1cmVkIElEIENBLTEwggEiMA0G
# CSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQDogi2Z+crCQpWlgHNAcNKeVlRcqcTS
# QQaPyTP8TUWRXIGf7Syc+BZZ3561JBXCmLm0d0ncicQK2q/LXmvtrbBxMevPOkAM
# Rk2T7It6NggDqww0/hhJgv7HxzFIgHweog+SDlDJxofrNj/YMMP/pvf7os1vcyP+
# rFYFkPAyIRaJxnCI+QWXfaPHQ90C6Ds97bFBo+0/vtuVSMTuHrPyvAwrmdDGXRJC
# geGDboJzPyZLFJCuWWYKxI2+0s4Grq2Eb0iEm09AufFM8q+Y+/bOQF1c9qjxL6/s
# iSLyaxhlscFzrdfx2M8eCnRcQrhofrfVdwonVnwPYqQ/MhRglf0HBKIJAgMBAAGj
# ggN6MIIDdjAOBgNVHQ8BAf8EBAMCAYYwOwYDVR0lBDQwMgYIKwYBBQUHAwEGCCsG
# AQUFBwMCBggrBgEFBQcDAwYIKwYBBQUHAwQGCCsGAQUFBwMIMIIB0gYDVR0gBIIB
# yTCCAcUwggG0BgpghkgBhv1sAAEEMIIBpDA6BggrBgEFBQcCARYuaHR0cDovL3d3
# dy5kaWdpY2VydC5jb20vc3NsLWNwcy1yZXBvc2l0b3J5Lmh0bTCCAWQGCCsGAQUF
# BwICMIIBVh6CAVIAQQBuAHkAIAB1AHMAZQAgAG8AZgAgAHQAaABpAHMAIABDAGUA
# cgB0AGkAZgBpAGMAYQB0AGUAIABjAG8AbgBzAHQAaQB0AHUAdABlAHMAIABhAGMA
# YwBlAHAAdABhAG4AYwBlACAAbwBmACAAdABoAGUAIABEAGkAZwBpAEMAZQByAHQA
# IABDAFAALwBDAFAAUwAgAGEAbgBkACAAdABoAGUAIABSAGUAbAB5AGkAbgBnACAA
# UABhAHIAdAB5ACAAQQBnAHIAZQBlAG0AZQBuAHQAIAB3AGgAaQBjAGgAIABsAGkA
# bQBpAHQAIABsAGkAYQBiAGkAbABpAHQAeQAgAGEAbgBkACAAYQByAGUAIABpAG4A
# YwBvAHIAcABvAHIAYQB0AGUAZAAgAGgAZQByAGUAaQBuACAAYgB5ACAAcgBlAGYA
# ZQByAGUAbgBjAGUALjALBglghkgBhv1sAxUwEgYDVR0TAQH/BAgwBgEB/wIBADB5
# BggrBgEFBQcBAQRtMGswJAYIKwYBBQUHMAGGGGh0dHA6Ly9vY3NwLmRpZ2ljZXJ0
# LmNvbTBDBggrBgEFBQcwAoY3aHR0cDovL2NhY2VydHMuZGlnaWNlcnQuY29tL0Rp
# Z2lDZXJ0QXNzdXJlZElEUm9vdENBLmNydDCBgQYDVR0fBHoweDA6oDigNoY0aHR0
# cDovL2NybDMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9vdENBLmNy
# bDA6oDigNoY0aHR0cDovL2NybDQuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJl
# ZElEUm9vdENBLmNybDAdBgNVHQ4EFgQUFQASKxOYspkH7R7for5XDStnAs0wHwYD
# VR0jBBgwFoAUReuir/SSy4IxLVGLp6chnfNtyA8wDQYJKoZIhvcNAQEFBQADggEB
# AEZQPsm3KCSnOB22WymvUs9S6TFHq1Zce9UNC0Gz7+x1H3Q48rJcYaKclcNQ5IK5
# I9G6OoZyrTh4rHVdFxc0ckeFlFbR67s2hHfMJKXzBBlVqefj56tizfuLLZDCwNK1
# lL1eT7EF0g49GqkUW6aGMWKoqDPkmzmnxPXOHXh2lCVz5Cqrz5x2S+1fwksW5Etw
# TACJHvzFebxMElf+X+EevAJdqP77BzhPDcZdkbkPZ0XN1oPt55INjbFpjE/7WeAj
# D9KqrgB87pxCDs+R1ye3Fu4Pw718CqDuLAhVhSK46xgaTfwqIa1JMYNHlXdx3LEb
# S0scEJx3FMGdTy9alQgpECYxggRQMIIETAIBATCBijB2MQswCQYDVQQGEwJVUzEV
# MBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29t
# MTUwMwYDVQQDEyxEaWdpQ2VydCBTSEEyIEhpZ2ggQXNzdXJhbmNlIENvZGUgU2ln
# bmluZyBDQQIQDTbwnKqD2aiMR2/WQV6y7zANBglghkgBZQMEAgEFAKCBhDAYBgor
# BgEEAYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJAzEMBgorBgEEAYI3AgEE
# MBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEVMC8GCSqGSIb3DQEJBDEiBCBv
# CptLaWq0TFWhw0RqGRXth0pRHI/gnWEGiX6JyQ7YxzANBgkqhkiG9w0BAQEFAASC
# AQCWxcHOG9GtXsRpWQAVGTouc/WsxL02eMwSnTAyrFYYkBh6K0MP+1cwnphiqOHr
# i6bHWZCHrGOggRVzA4FbPZeSgCVwnQVuPws4N5zi7sBGMy3BtDDLjFUoMBNHImei
# o3kYK2LwsFlWMdHIZkUkGbQGuVrfJ18gSXGaUKLYKIKs79CXRt8Ay8qka0z8Poor
# jLFHnInzABFg9ITEGFSKVWfQEEXlWdjhbEViJp69nG20kWxQzsEO+1WK+QpAIc4B
# I/+6vonW1cqo+wAaerJNQROsGTc3iXCjwtSW/vfJPpp1EId2C41jpAxQH+Hy/QuR
# 0z2AlDp9Nrkf1OXJVthB2SlwoYICDzCCAgsGCSqGSIb3DQEJBjGCAfwwggH4AgEB
# MHYwYjELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UE
# CxMQd3d3LmRpZ2ljZXJ0LmNvbTEhMB8GA1UEAxMYRGlnaUNlcnQgQXNzdXJlZCBJ
# RCBDQS0xAhADAZoCOv9YsWvW1ermF/BmMAkGBSsOAwIaBQCgXTAYBgkqhkiG9w0B
# CQMxCwYJKoZIhvcNAQcBMBwGCSqGSIb3DQEJBTEPFw0yMDA0MjEwMTA4MjdaMCMG
# CSqGSIb3DQEJBDEWBBRpBWg/Ct4UaWstPGCRpz+Jf5K7uTANBgkqhkiG9w0BAQEF
# AASCAQAyJ8r2oKE+QDBrxhy6Wgw10qy8RawHDLYSHiSdaiM6r6+d+an5L7vrCQx4
# tJTtKRr5NhuY2A/QOgkYzjpN6j7NOT2CKnHBZQldIN3eAPSuP2fqAkkQPZ+mah+f
# KlLtG9tI5xiAvsTFp0wh1dFSEmk1CxkE8HaLVct782/bRh9Wir+jgkr4KkZGrkyh
# TEElTHBwbmbeMFcH62zgVC6Ibk8fD4niGxx7wQjUekYoPQHJ7zXvDNExQfNS6ZlP
# CIr7vH+yvpY0bs5ODDj+ULdtcuXpZFhwVivAWaDehMDe0JLkBYOgaiFxEtzapcU9
# ZSpCFoGhj25vCPEfx3oXDlFbvGe1
# SIG # End signature block
